@@ -4,8 +4,8 @@
  * @brief netopeer2-cli commands header
  *
  * @copyright
- * Copyright (c) 2019 - 2021 Deutsche Telekom AG.
- * Copyright (c) 2017 - 2021 CESNET, z.s.p.o.
+ * Copyright (c) 2019 - 2023 Deutsche Telekom AG.
+ * Copyright (c) 2017 - 2023 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -17,7 +17,7 @@
 #ifndef COMMANDS_H_
 #define COMMANDS_H_
 
-#include "cli_version.h"
+#include "cli_config.h"
 
 #include <stdio.h>
 
@@ -25,10 +25,9 @@ extern char some_msg[];
 #define INSTRUCTION(format, args ...) {snprintf(some_msg,4095,format,##args);printf("\n  %s",some_msg);}
 #define ERROR(function, format, args ...) {snprintf(some_msg,4095,format,##args);fprintf(stderr,"%s: %s\n",function,some_msg);}
 
-#define PROMPT "> "
-
 typedef struct {
     char *name; /* User printable name of the function. */
+
     int (*func)(const char *, char **); /* Function to call to do the command. */
     void (*help_func)(void); /* Display command help. */
     char *helpstring; /* Documentation for this function. */
