@@ -4,8 +4,8 @@
  * @brief netopeer2-cli auto completion
  *
  * @copyright
- * Copyright (c) 2019 - 2021 Deutsche Telekom AG.
- * Copyright (c) 2017 - 2021 CESNET, z.s.p.o.
+ * Copyright (c) 2019 - 2023 Deutsche Telekom AG.
+ * Copyright (c) 2017 - 2023 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -35,10 +35,72 @@
 
 #include "commands.h"
 #include "compat.h"
+#include "configuration.h"
 #include "linenoise/linenoise.h"
 
-extern struct ly_ctx *ctx;
-extern char *config_editor;
+/**
+ * @brief Fill path completion.
+ *
+ * @param[in] hint Path to directory.
+ * @param[in] lc For adding completions.
+ */
+static void
+path_completion(const char *hint, linenoiseCompletions *lc)
+{
+    const char *ptr;
+    char *full_path, *hint_ptr, match[FILENAME_MAX + 2];
+    DIR *dir;
+    struct dirent *ent;
+    struct stat st;
+
+    lc->path = 1;
+
+    ptr = strrchr(hint, '/');
+
+    /* new relative path */
+    if (ptr == NULL) {
+        full_path = malloc(2 + FILENAME_MAX + 1);
+        strcpy(full_path, "./");
+
+        ptr = hint;
+    } else {
+        full_path = malloc((int)(ptr - hint) + FILENAME_MAX + 1);
+        ++ptr;
+        sprintf(full_path, "%.*s", (int)(ptr - hint), hint);
+    }
+    hint_ptr = full_path + strlen(full_path);
+
+    dir = opendir(full_path);
+    if (dir == NULL) {
+        free(full_path);
+        return;
+    }
+
+    while ((ent = readdir(dir))) {
+        if (ent->d_name[0] == '.') {
+            continue;
+        }
+
+        if (!strncmp(ptr, ent->d_name, strlen(ptr))) {
+            /* is it a directory? */
+            strcpy(hint_ptr, ent->d_name);
+            if (stat(full_path, &st)) {
+                /* skip this item */
+                continue;
+            }
+
+            strcpy(match, ent->d_name);
+            if (S_ISDIR(st.st_mode)) {
+                strcat(match, "/");
+            }
+
+            linenoiseAddCompletion(lc, match);
+        }
+    }
+
+    free(full_path);
+    closedir(dir);
+}
 
 static void
 get_cmd_completion(const char *hint, char ***matches, unsigned int *match_count)
@@ -84,26 +146,23 @@ complete_cmd(const char *buf, const char *hint, linenoiseCompletions *lc)
     unsigned int match_count = 0, i;
 
     if (!strncmp(buf, "searchpath ", 11)
-#ifdef NC_ENABLED_SSH
-            || !strncmp(buf, "auth keys add ", 14)
-#endif
-#ifdef NC_ENABLED_TLS
-            || !strncmp(buf, "cert add ", 9) || !strncmp(buf, "cert remove ", 12) || !strncmp(buf, "cert replaceown ", 16) ||
-            !strncmp(buf, "crl add ", 8) || !strncmp(buf, "crl remove ", 11)
+#ifdef NC_ENABLED_SSH_TLS
+            || !strncmp(buf, "auth keys add ", 14) || !strncmp(buf, "cert add ", 9) ||
+            !strncmp(buf, "cert remove ", 12) || !strncmp(buf, "cert replaceown ", 16)
 #endif
        ) {
-        linenoisePathCompletion(buf, hint, lc);
+        path_completion(hint, lc);
     } else if ((!strncmp(buf, "copy-config ", 12) || !strncmp(buf, "validate ", 9)) && last_opt(buf, hint, "--src-config")) {
-        linenoisePathCompletion(buf, hint, lc);
+        path_completion(hint, lc);
     } else if (!strncmp(buf, "edit-config ", 12) && last_opt(buf, hint, "--config")) {
-        linenoisePathCompletion(buf, hint, lc);
+        path_completion(hint, lc);
     } else if ((!strncmp(buf, "get ", 4) || !strncmp(buf, "get-config ", 11) || !strncmp(buf, "subscribe ", 10)) &&
             (last_opt(buf, hint, "--filter-subtree") || last_opt(buf, hint, "--out"))) {
-        linenoisePathCompletion(buf, hint, lc);
+        path_completion(hint, lc);
     } else if (!strncmp(buf, "get-schema ", 11) && last_opt(buf, hint, "--out")) {
-        linenoisePathCompletion(buf, hint, lc);
+        path_completion(hint, lc);
     } else if (!strncmp(buf, "user-rpc ", 9) && last_opt(buf, hint, "--content")) {
-        linenoisePathCompletion(buf, hint, lc);
+        path_completion(hint, lc);
     } else if (!strchr(buf, ' ') && hint[0]) {
         get_cmd_completion(hint, &matches, &match_count);
 
@@ -201,7 +260,7 @@ readinput(const char *instruction, const char *old_tmp, char **new_tmp)
         goto fail;
     } else if (pid == 0) {
         /* child */
-        execlp(config_editor, config_editor, tmpname, (char *)NULL);
+        execlp(opts.config_editor, opts.config_editor, tmpname, (char *)NULL);
 
         ERROR(__func__, "Exec failed (%s).", strerror(errno));
         exit(1);
@@ -264,6 +323,8 @@ readinput(const char *instruction, const char *old_tmp, char **new_tmp)
         }
     }
 
+cleanup:
+
     if (new_tmp) {
         *new_tmp = tmpname;
     } else {
@@ -271,8 +332,6 @@ readinput(const char *instruction, const char *old_tmp, char **new_tmp)
         free(tmpname);
     }
 
-cleanup:
-
     close(tmpfd);
     free(old_content);
 
