@@ -1,11 +1,12 @@
 /**
  * @file netconf_confirmed_commit.c
  * @author Tadeas Vintrlik <xvintr04@stud.fit.vutbr.cz>
+ * @author Michal Vasko <mvasko@cesnet.cz>
  * @brief ietf-netconf confirmed-commit capability callbacks
  *
  * @copyright
- * Copyright (c) 2019 - 2021 Deutsche Telekom AG.
- * Copyright (c) 2017 - 2021 CESNET, z.s.p.o.
+ * Copyright (c) 2019 - 2023 Deutsche Telekom AG.
+ * Copyright (c) 2017 - 2023 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -34,6 +35,7 @@
 #include <libyang/libyang.h>
 #include <nc_server.h>
 #include <sysrepo.h>
+#include <sysrepo/netconf_acm.h>
 
 #include "common.h"
 #include "compat.h"
@@ -50,13 +52,12 @@
  * When accessing struct members must use the lock mutex.
  * Alternatively use the set_* and get_* functions
  */
-typedef struct commit_ctx_s {
-    char *persist;        /* What persist-id is expected */
-    timer_t timer;        /* POSIX timer used for rollback, zero if none */
-    pthread_mutex_t lock; /* Lock mutexing this structure and access to NCC_DIR */
-} commit_ctx_t;
-
-static commit_ctx_t commit_ctx = {.persist = NULL, .timer = 0, .lock = PTHREAD_MUTEX_INITIALIZER};
+static struct {
+    char *persist;              /* persist-id of the commit */
+    struct nc_session *nc_sess; /* NETCONF session of the pending commit */
+    timer_t timer;              /* POSIX timer used for rollback, zero if none */
+    pthread_mutex_t lock;       /* Lock for access to this structure and to NCC_DIR */
+} commit_ctx = {.persist = NULL, .timer = 0, .lock = PTHREAD_MUTEX_INITIALIZER};
 
 void
 ncc_commit_ctx_destroy(void)
@@ -68,6 +69,27 @@ ncc_commit_ctx_destroy(void)
     commit_ctx.persist = NULL;
 }
 
+int
+ncc_ongoing_confirmed_commit(struct nc_session **nc_sess)
+{
+    int cc;
+
+    /* LOCK */
+    pthread_mutex_lock(&commit_ctx.lock);
+
+    if (commit_ctx.timer) {
+        *nc_sess = commit_ctx.nc_sess;
+        cc = 1;
+    } else {
+        cc = 0;
+    }
+
+    /* UNLOCK */
+    pthread_mutex_unlock(&commit_ctx.lock);
+
+    return cc;
+}
+
 /**
  * @brief Set value of persist in the commit context structure.
  *
@@ -106,7 +128,7 @@ cleanup:
  * @return Module name.
  */
 static char *
-get_module_name_from_filename(const char *filename)
+ncc_get_module_name_from_filename(const char *filename)
 {
     char *point = strstr(filename, ".json");
     char *new = NULL;
@@ -132,11 +154,11 @@ cleanup:
  * @param[in] path Absolute path to the file.
  */
 static void
-rename_failed_file(const char *filename, const char *path)
+ncc_rename_failed_file(const char *filename, const char *path)
 {
     char *new = NULL;
 
-    if (asprintf(&new, "%s/%s/%s-%ld.failed", np2srv.server_dir, NCC_DIR, filename, time(NULL)) == -1) {
+    if (asprintf(&new, "%s/%s/%s-%lld.failed", np2srv.server_dir, NCC_DIR, filename, (long long)time(NULL)) == -1) {
         EMEM;
         return;
     }
@@ -160,12 +182,12 @@ cleanup:
  * @return SR_ERR_OK When successful.
  */
 static int
-get_running_backup(const struct ly_ctx *ctx, const char *path, struct lyd_node **node)
+ncc_get_running_backup(const struct ly_ctx *ctx, const char *path, struct lyd_node **node)
 {
     int ret = SR_ERR_OK;
 
     if (lyd_parse_data_path(ctx, path, LYD_JSON, LYD_PARSE_ORDERED | LYD_PARSE_STRICT | LYD_PARSE_ONLY, 0, node)) {
-        ERR("Failed parsing confirmed commit backup of running for file \"%s\" (%s).", path, ly_errmsg(ctx));
+        ERR("Failed parsing confirmed commit backup of running for file \"%s\" (%s).", path, ly_err_last(ctx)->msg);
         ret = SR_ERR_LY;
         goto cleanup;
     }
@@ -260,7 +282,7 @@ cleanup:
  * @return SR_ERR_OK When directory exists or was successfully created.
  */
 static int
-ncc_check_dir(void)
+ncc_check_server_dir(void)
 {
     int rc = SR_ERR_OK;
     char *path = NULL;
@@ -302,117 +324,11 @@ cleanup:
     return rc;
 }
 
-/**
- * @brief Restore running using the backup files.
- * Thread run after the timer in commit_ctx_s runs out.
- */
-static void
-changes_rollback(union sigval UNUSED(sev))
-{
-    int rc;
-    struct lyd_node *node = NULL;
-    const struct ly_ctx *ctx = NULL;
-    struct lys_module *module = NULL;
-    sr_session_ctx_t *session;
-    char *path = NULL, *module_name = NULL, *meta = NULL, *srv_path = NULL;
-    uint32_t nc_id;
-    DIR *dir = NULL;
-    struct dirent *dirent = NULL;
-
-    VRB("Confirmed commit timeout reached. Restoring previous running.");
-    ctx = sr_get_context(np2srv.sr_conn);
-
-    /* Start a session */
-    if ((rc = sr_session_start(np2srv.sr_conn, SR_DS_RUNNING, &session))) {
-        ERR("Failed starting a sysrepo session (%s).", sr_strerror(rc));
-        goto cleanup;
-    }
-    /* set session attributes for diff_check_cb to skip NACM check */
-    sr_session_set_orig_name(session, "netopeer2");
-    /* nc_id */
-    nc_id = 0;
-    sr_session_push_orig_data(session, sizeof nc_id, &nc_id);
-    /* username */
-    sr_session_push_orig_data(session, 1, "");
-
-    if ((rc = ncc_check_dir())) {
-        goto cleanup;
-    }
-    /* Iterate over all files in backup directory */
-    if (asprintf(&srv_path, "%s/%s", np2srv.server_dir, NCC_DIR) == -1) {
-        EMEM;
-        goto cleanup;
-    }
-    dir = opendir(srv_path);
-    if (!dir) {
-        ERR("Failed opening netopeer2 server directory \"%s\".", srv_path);
-        goto cleanup;
-    }
-    while ((dirent = readdir(dir))) {
-        if (!strcmp(dirent->d_name, ".") || !strcmp(dirent->d_name, "..") || !strcmp(dirent->d_name, META_FILE)) {
-            continue;
-        }
-
-        /* try to find the module that corresponds with the file */
-        free(module_name);
-        module_name = get_module_name_from_filename(dirent->d_name);
-        if (!module_name) {
-            /* Skipping files that do not match the expected format */
-            continue;
-        }
-        free(path);
-        path = NULL;
-        if (asprintf(&path, "%s/%s/%s", np2srv.server_dir, NCC_DIR, dirent->d_name) == -1) {
-            EMEM;
-            goto cleanup;
-        }
-        module = ly_ctx_get_module_implemented(ctx, module_name);
-        if (!module) {
-            ERR("Module \"%s\" does not exist/not implemented.", module_name);
-            rename_failed_file(module_name, path);
-            continue;
-        }
-
-        /* get, restore and delete the backup */
-        VRB("Rolling back module \"%s\"", module->name);
-        if (get_running_backup(ctx, path, &node)) {
-            rename_failed_file(module_name, path);
-            continue;
-        }
-        if ((rc = sr_replace_config(session, module->name, node, np2srv.sr_timeout))) {
-            ERR("Failed restoring backup for module \"%s\".", module->name);
-            rename_failed_file(module_name, path);
-            continue;
-        }
-        if (unlink(path) == -1) {
-            ERR("Failed removing backup file \"%s\" (%s).", path, strerror(errno));
-            goto cleanup;
-        }
-    }
-
-    if (asprintf(&meta, "%s/%s/%s", np2srv.server_dir, NCC_DIR, META_FILE) < 0) {
-        EMEM;
-        goto cleanup;
-    }
-    if (unlink(meta) == -1) {
-        ERR("Failed removing confirmed commit meta file (%s).", strerror(errno));
-        goto cleanup;
-    }
-
-cleanup:
-    closedir(dir);
-    sr_session_stop(session);
-    free(path);
-    free(srv_path);
-    free(meta);
-    free(module_name);
-}
-
 /**
  * @brief Remove all the backup files not marked as failed.
  */
 static void
-clean_backup_directory(void)
+ncc_clean_backup_directory(void)
 {
     DIR *dir = NULL;
     struct dirent *dirent;
@@ -429,8 +345,11 @@ clean_backup_directory(void)
         goto cleanup;
     }
     while ((dirent = readdir(dir))) {
-        if (!strcmp(dirent->d_name, ".") || !strcmp(dirent->d_name, "..") ||
-                !strstr(".json", dirent->d_name) || strcmp(META_FILE, dirent->d_name)) {
+        if (!strcmp(dirent->d_name, ".") || !strcmp(dirent->d_name, "..")) {
+            continue;
+        }
+
+        if (!strstr(dirent->d_name, ".json") && strcmp(META_FILE, dirent->d_name)) {
             /* If some unexpected file, just skip */
             continue;
         }
@@ -460,19 +379,175 @@ cleanup:
 static void
 ncc_commit_confirmed(void)
 {
-    timer_delete(commit_ctx.timer);
+    if (commit_ctx.timer) {
+        timer_delete(commit_ctx.timer);
+    }
     commit_ctx.timer = 0;
-    clean_backup_directory();
+    commit_ctx.nc_sess = NULL;
+    ncc_set_persist(NULL);
+    ncc_clean_backup_directory();
 }
 
 /**
- * @brief Cancel pending commit. Rollback running from backup.
+ * @brief Restore running using the backup files.
+ * Thread run after the timer in commit_ctx_s runs out.
+ *
+ * @param[in] sev Received signal. 0 when called by the timer (no commit lock).
  */
 static void
-ncc_commit_cancel(void)
+ncc_changes_rollback_cb(union sigval sev)
 {
-    changes_rollback((union sigval)0);
+    int rc;
+    struct np2_user_sess *user_sess = NULL;
+    sr_session_ctx_t *sr_sess = NULL;
+    struct lyd_node *node = NULL;
+    const struct ly_ctx *ly_ctx;
+    struct lys_module *module = NULL;
+    char *path = NULL, *module_name = NULL, *meta = NULL, *srv_path = NULL, *nacm_user = NULL;
+    DIR *dir = NULL;
+    struct dirent *dirent = NULL;
+
+    /* basic server dir check */
+    if (ncc_check_server_dir()) {
+        return;
+    }
+    ly_ctx = sr_acquire_context(np2srv.sr_conn);
+
+    if (!sev.sival_int) {
+        /* LOCK */
+        pthread_mutex_lock(&commit_ctx.lock);
+    }
+
+    if (commit_ctx.nc_sess) {
+        /* get user session, use the original NC (SR) session in case it was used for locking running DS */
+        if ((rc = np_acquire_user_sess(commit_ctx.nc_sess, &user_sess))) {
+            goto cleanup;
+        }
+        sr_sess = user_sess->sess;
+
+        /* clear user to skip NACM checks */
+        nacm_user = strdup(sr_nacm_get_user(sr_sess));
+        if (!nacm_user) {
+            EMEM;
+            goto cleanup;
+        }
+        sr_nacm_set_user(sr_sess, NULL);
+
+        /* replacing running datastore */
+        sr_session_switch_ds(sr_sess, SR_DS_RUNNING);
+    } else {
+        /* create a new SR session for the rollback */
+        if ((rc = sr_session_start(np2srv.sr_conn, SR_DS_RUNNING, &sr_sess))) {
+            goto cleanup;
+        }
+    }
+
+    /* iterate over all the files in backup directory */
+    if (asprintf(&srv_path, "%s/%s", np2srv.server_dir, NCC_DIR) == -1) {
+        EMEM;
+        goto cleanup;
+    }
+    dir = opendir(srv_path);
+    if (!dir) {
+        ERR("Failed opening netopeer2 server directory \"%s\".", srv_path);
+        goto cleanup;
+    }
+    while ((dirent = readdir(dir))) {
+        if (!strcmp(dirent->d_name, ".") || !strcmp(dirent->d_name, "..") || !strcmp(dirent->d_name, META_FILE)) {
+            continue;
+        }
+
+        /* try to find the module that corresponds with the file */
+        free(module_name);
+        module_name = ncc_get_module_name_from_filename(dirent->d_name);
+        if (!module_name) {
+            /* Skipping files that do not match the expected format */
+            continue;
+        }
+        free(path);
+        path = NULL;
+        if (asprintf(&path, "%s/%s/%s", np2srv.server_dir, NCC_DIR, dirent->d_name) == -1) {
+            EMEM;
+            goto cleanup;
+        }
+        module = ly_ctx_get_module_implemented(ly_ctx, module_name);
+        if (!module) {
+            ERR("Module \"%s\" does not exist/not implemented.", module_name);
+            ncc_rename_failed_file(module_name, path);
+            continue;
+        }
+
+        /* get, restore, and delete the backup */
+        VRB("Rolling back module \"%s\"...", module->name);
+        if (ncc_get_running_backup(ly_ctx, path, &node)) {
+            ncc_rename_failed_file(module_name, path);
+            continue;
+        }
+        if ((rc = sr_replace_config(sr_sess, module->name, node, np2srv.sr_timeout))) {
+            ERR("Failed restoring backup for module \"%s\".", module->name);
+            ncc_rename_failed_file(module_name, path);
+            continue;
+        }
+        if (unlink(path) == -1) {
+            ERR("Failed removing backup file \"%s\" (%s).", path, strerror(errno));
+            goto cleanup;
+        }
+    }
+
+    if (asprintf(&meta, "%s/%s/%s", np2srv.server_dir, NCC_DIR, META_FILE) < 0) {
+        EMEM;
+        goto cleanup;
+    }
+    if (unlink(meta) == -1) {
+        ERR("Failed removing confirmed commit meta file (%s).", strerror(errno));
+        goto cleanup;
+    }
+
+    /* just timer clean up */
     ncc_commit_confirmed();
+
+cleanup:
+    if (!sev.sival_int) {
+        /* UNLOCK */
+        pthread_mutex_unlock(&commit_ctx.lock);
+
+        /* send notification about timeout for confirmed-commits */
+        np_send_notif_confirmed_commit(commit_ctx.nc_sess, sr_sess, NP_CC_TIMEOUT, 0, 0);
+    }
+    sr_release_context(np2srv.sr_conn);
+    if (user_sess) {
+        if (nacm_user) {
+            sr_nacm_set_user(sr_sess, nacm_user);
+            free(nacm_user);
+        }
+        np_release_user_sess(user_sess);
+    } else {
+        sr_session_stop(sr_sess);
+    }
+    closedir(dir);
+    free(path);
+    free(srv_path);
+    free(meta);
+    free(module_name);
+}
+
+void
+ncc_del_session(const struct nc_session *nc_sess, sr_session_ctx_t *sr_sess)
+{
+    /* LOCK */
+    pthread_mutex_lock(&commit_ctx.lock);
+
+    if (commit_ctx.timer && !commit_ctx.persist && (commit_ctx.nc_sess == nc_sess)) {
+        /* rollback */
+        VRB("Performing confirmed commit rollback after the issuing session has terminated.");
+        ncc_changes_rollback_cb((union sigval)1);
+
+        /* send notification about canceling confirmed-commits */
+        np_send_notif_confirmed_commit(nc_sess, sr_sess, NP_CC_CANCEL, 0, 0);
+    }
+
+    /* UNLOCK */
+    pthread_mutex_unlock(&commit_ctx.lock);
 }
 
 /**
@@ -480,17 +555,16 @@ ncc_commit_cancel(void)
  *
  * @param[in] session Sysrepo session used to get data of the module.
  * @param[in] module Module to backup.
- *
  * @return SR_ERR_LY When printing into the file failed.
  * @return SR_ERR_NO_MEMORY When memory ran during allocation
  * @return SR_ERR_OK On success
  */
 static int
-backup_module(sr_session_ctx_t *session, const struct lys_module *module)
+ncc_backup_module(sr_session_ctx_t *session, const struct lys_module *module)
 {
     int rc = SR_ERR_OK;
     char *path = NULL, *xpath = NULL, *ncc_path = NULL;
-    struct lyd_node *node = NULL;
+    sr_data_t *data = NULL;
 
     if (asprintf(&ncc_path, "%s/%s", np2srv.server_dir, NCC_DIR) == -1) {
         EMEM;
@@ -505,7 +579,7 @@ backup_module(sr_session_ctx_t *session, const struct lys_module *module)
         goto cleanup;
     }
 
-    if ((rc = sr_get_data(session, xpath, 0, 0, 0, &node))) {
+    if ((rc = sr_get_data(session, xpath, 0, 0, 0, &data))) {
         ERR("Failed getting configuration of running for module \"%s\" (%s).", module->name, sr_strerror(rc));
         goto cleanup;
     }
@@ -515,15 +589,15 @@ backup_module(sr_session_ctx_t *session, const struct lys_module *module)
         rc = SR_ERR_NO_MEMORY;
         goto cleanup;
     }
-    if (lyd_print_path(path, node, LYD_JSON, LY_PRINT_SHRINK)) {
-        ERR("Failed backing up node of module \"%s\" into file \"%s\" (%s).",
-                module->name, path, ly_errmsg(LYD_CTX(node)));
+    if (lyd_print_path(path, data ? data->tree : NULL, LYD_JSON, LYD_PRINT_WITHSIBLINGS | LYD_PRINT_SHRINK)) {
+        ERR("Failed backing up node of module \"%s\" into file \"%s\".", module->name, path);
         rc = SR_ERR_LY;
         goto cleanup;
     }
 
 cleanup:
-    lyd_free_tree(node);
+    sr_release_data(data);
+    free(ncc_path);
     free(path);
     free(xpath);
     return rc;
@@ -532,10 +606,9 @@ cleanup:
 /**
  * @brief Schedule a rollback of confirmed commit. Create the timer and set all the options.
  *
- * @param[in] timeout_s Time (in seconnds) after which the timer will start the rollback.
- *
- * @return SR_ERR_SYS When creating the timer fails.
- * @return SR_ERR_OK When succeeded.
+ * @param[in] timeout_s Time (in seconds) after which the timer will start the rollback.
+ * @return SR_ERR_SYS on timer creation failure.
+ * @return SR_ERR_OK on succeeded.
  */
 static int
 ncc_commit_timeout_schedule(uint32_t timeout_s)
@@ -544,19 +617,29 @@ ncc_commit_timeout_schedule(uint32_t timeout_s)
     struct itimerspec its = {0};
     timer_t timer_id;
 
-    /* create and arm the timer */
-    sev.sigev_notify = SIGEV_THREAD;
-    sev.sigev_notify_function = changes_rollback;
-    its.it_value.tv_sec = timeout_s;
-    if (timer_create(CLOCK_REALTIME, &sev, &timer_id) == -1) {
-        ERR("Could not create a timer for confirmed commit rollback (%s).", strerror(errno));
-        return SR_ERR_SYS;
+    assert(!commit_ctx.timer);
+
+    VRB("Scheduling confirmed commit rollback in %" PRIu32 "s.", timeout_s);
+
+    if (!timeout_s) {
+        /* just perform the rollback without locking */
+        ncc_changes_rollback_cb((union sigval)1);
+    } else {
+        /* create and arm the timer */
+        sev.sigev_notify = SIGEV_THREAD;
+        sev.sigev_value = (union sigval)0;
+        sev.sigev_notify_function = ncc_changes_rollback_cb;
+        its.it_value.tv_sec = timeout_s;
+        if (timer_create(CLOCK_REALTIME, &sev, &timer_id) == -1) {
+            ERR("Could not create a timer for confirmed commit rollback (%s).", strerror(errno));
+            return SR_ERR_SYS;
+        }
+        if (timer_settime(timer_id, 0, &its, NULL) == -1) {
+            ERR("Could not set time in timer for confirmed commit rollback (%s).", strerror(errno));
+            return SR_ERR_SYS;
+        }
+        commit_ctx.timer = timer_id;
     }
-    if (timer_settime(timer_id, 0, &its, NULL) == -1) {
-        ERR("Could not set time in timer for confirmed commit rollback (%s).", strerror(errno));
-        return SR_ERR_SYS;
-    }
-    commit_ctx.timer = timer_id;
 
     return SR_ERR_OK;
 }
@@ -570,7 +653,7 @@ ncc_commit_timeout_schedule(uint32_t timeout_s)
  * @return SR_ERR_OK When successful. If time is zero no meta file existed.
  */
 static int
-read_meta_file(time_t *time, uint32_t *timeout_s)
+ncc_read_meta_file(time_t *time, uint32_t *timeout_s)
 {
     int rc = SR_ERR_OK;
     FILE *file = NULL;
@@ -614,30 +697,21 @@ cleanup:
 void
 ncc_try_restore(void)
 {
-    time_t timestamp = 0, end_time = 0, current = 0;
-    uint32_t timeout = 0, new_timeout = 0;
+    time_t timestamp = 0;
+    uint32_t timeout = 0;
 
-    /* In theory it should be under a mutex, but since it is called in init it is not needed */
+    /* it should be under a mutex, but since it is called in init it is not needed */
 
-    if (read_meta_file(&timestamp, &timeout)) {
+    if (ncc_read_meta_file(&timestamp, &timeout)) {
         return;
     }
     if (!timestamp) {
-        /* No meta file existed */
+        /* no meta file existed */
         return;
     }
 
-    /* Check when the confirmed commit was supposed to timeout */
-    end_time = timestamp + timeout;
-    current = time(NULL);
-    if (end_time > current) {
-        /* In the future -> compute the remaining time */
-        new_timeout = end_time - current;
-    }
-    /* else it was in the past -> should be zero */
-
-    VRB("Restoring a previous confirmed commit.");
-    ncc_commit_timeout_schedule(new_timeout);
+    VRB("Performing confirmed commit rollback after server restart.");
+    ncc_changes_rollback_cb((union sigval)1);
 }
 
 /**
@@ -646,27 +720,26 @@ ncc_try_restore(void)
  * @return SR_ERR_OK When successful.
  */
 static int
-set_running_backup(void)
+ncc_running_backup(void)
 {
     int rc = SR_ERR_OK, read = 0, write = 0;
-    const struct ly_ctx *ctx;
-    struct sr_session_ctx_s *session;
+    const struct ly_ctx *ly_ctx;
+    sr_session_ctx_t *session;
     struct lys_module *module;
-    sr_conn_ctx_t *conn;
     uint32_t index = 0;
 
+    ly_ctx = sr_acquire_context(np2srv.sr_conn);
+
     if ((rc = sr_session_start(np2srv.sr_conn, SR_DS_RUNNING, &session))) {
         ERR("Failed starting a sysrepo session (%s).", sr_strerror(rc));
         goto cleanup;
     }
 
-    /* Iterate over all implemented modules */
-    conn = sr_session_get_connection(session);
-    ctx = sr_get_context(conn);
-    if ((rc = ncc_check_dir())) {
+    /* iterate over all implemented modules */
+    if ((rc = ncc_check_server_dir())) {
         goto cleanup;
     }
-    while ((module = ly_ctx_get_module_iter(ctx, &index))) {
+    while ((module = ly_ctx_get_module_iter(ly_ctx, &index))) {
         /* check if module should and can be backed up */
         if (!module->implemented) {
             continue;
@@ -674,9 +747,12 @@ set_running_backup(void)
         if (!np_ly_mod_has_data(module, LYS_CONFIG_W)) {
             continue;
         }
+        if (!strcmp(module->name, "sysrepo")) {
+            continue;
+        }
 
         /* Check if has both read and write permission for module in sysrepo */
-        if ((rc = sr_check_module_ds_access(conn, "edit1", SR_DS_RUNNING, &read, &write))) {
+        if ((rc = sr_check_module_ds_access(np2srv.sr_conn, module->name, SR_DS_RUNNING, &read, &write))) {
             ERR("Failed getting permissions of module \"%s\".", module->name);
             goto cleanup;
         }
@@ -685,13 +761,14 @@ set_running_backup(void)
         }
 
         /* Create the backup */
-        if ((rc = backup_module(session, module))) {
+        if ((rc = ncc_backup_module(session, module))) {
             ERR("Failed creating backup of module \"%s\".", module->name);
             goto cleanup;
         }
     }
 
 cleanup:
+    sr_release_context(np2srv.sr_conn);
     sr_session_stop(session);
     return rc;
 }
@@ -702,8 +779,9 @@ cleanup:
  * @param[in] timeout_s Timeout (in seconds) that was used for the confirmed commit.
  */
 static void
-create_meta_file(uint32_t timeout_s)
+ncc_create_meta_file(uint32_t timeout_s)
 {
+    int fd = -1;
     FILE *file = NULL;
     char *meta;
 
@@ -711,14 +789,26 @@ create_meta_file(uint32_t timeout_s)
         EMEM;
         goto cleanup;
     }
-    file = fopen(meta, "w");
+
+    fd = open(meta, O_WRONLY | O_CREAT | O_TRUNC, 0600);
+    if (fd == -1) {
+        WRN("Failed creating confirmed commit meta file. Changes will not recover in case the server is stopped.");
+        goto cleanup;
+    }
+
+    file = fdopen(fd, "w");
     if (!file) {
         WRN("Failed creating confirmed commit meta file. Changes will not recover in case the server is stopped.");
         goto cleanup;
     }
+    fd = -1;
+
     fprintf(file, "%ld\n%" PRIu32 "\n", (long)time(NULL), timeout_s);
 
 cleanup:
+    if (fd > -1) {
+        close(fd);
+    }
     if (file) {
         fclose(file);
     }
@@ -737,15 +827,20 @@ static int
 np2srv_confirmed_commit_cb(sr_session_ctx_t *session, const struct lyd_node *input)
 {
     int rc = SR_ERR_OK;
-    struct np2_user_sess *user_sess;
+    struct np2_user_sess *user_sess = NULL;
+    struct nc_session *nc_sess;
     const sr_error_info_t *err_info;
     const char *persist = NULL;
     struct lyd_node *node = NULL;
     char *endptr = NULL;
     uint32_t timeout;
+    uint8_t timeout_changed = 0;
 
     /* get the user session */
-    if ((rc = np_get_user_sess(session, NULL, &user_sess))) {
+    if ((rc = np_find_user_sess(session, __func__, &nc_sess, &user_sess))) {
+        goto cleanup;
+    }
+    if ((rc = sr_session_switch_ds(user_sess->sess, SR_DS_RUNNING))) {
         goto cleanup;
     }
 
@@ -754,47 +849,79 @@ np2srv_confirmed_commit_cb(sr_session_ctx_t *session, const struct lyd_node *inp
     assert(node);
     timeout = strtoul(lyd_get_value(node), &endptr, 10);
     if (*endptr) {
-        rc = SR_ERR_INVAL_ARG;
         ERR("Invalid timeout \"%s\" given", lyd_get_value(node));
+        rc = SR_ERR_INVAL_ARG;
         goto cleanup;
     }
 
     /* persist */
-    lyd_find_path(input, "persist", 0, &node);
-    if (node) {
+    if (!lyd_find_path(input, "persist", 0, &node)) {
         persist = lyd_get_value(node);
     }
 
     /* persist-id */
-    lyd_find_path(input, "persist-id", 0, &node);
-    if (node) {
+    if (!lyd_find_path(input, "persist-id", 0, &node)) {
         ERR("Persist-id given in confirmed commit rpc.");
         rc = SR_ERR_INVAL_ARG;
         goto cleanup;
     }
 
-    /* create and store the backup */
-    if ((rc = sr_session_switch_ds(user_sess->sess, SR_DS_RUNNING))) {
-        goto cleanup;
-    }
-    if (set_running_backup()) {
-        goto cleanup;
-    }
-    create_meta_file(timeout);
+    if (!commit_ctx.timer) {
+        /* create and store the backup */
+        if (ncc_running_backup()) {
+            goto cleanup;
+        }
+    } else {
+        if (commit_ctx.persist) {
+            if (!persist || strcmp(persist, commit_ctx.persist)) {
+                np_err_invalid_value(session, "Follow-up confirm commit does not match pending confirmed commit.", "persist");
+                rc = SR_ERR_INVAL_ARG;
+                goto cleanup;
+            }
+        } else {
+            assert(commit_ctx.nc_sess);
+            if (commit_ctx.nc_sess != nc_sess) {
+                np_err_operation_failed(session,
+                        "Follow-up confirm commit session does not match the pending confirmed commit session.");
+                rc = SR_ERR_OPERATION_FAILED;
+                goto cleanup;
+            }
+        }
 
-    /* Set persist and start timer thread for rollback */
+        /* there is already a pending confirmed commit, keep its backup, but the timeout will be reset */
+        timer_delete(commit_ctx.timer);
+        commit_ctx.timer = 0;
+        timeout_changed = 1;
+    }
+
+    /* (re)set the meta file timeout */
+    ncc_create_meta_file(timeout);
+
+    /* set persist or NC session */
     if (persist) {
         if (ncc_set_persist(persist)) {
             goto cleanup;
         }
+    } else {
+        commit_ctx.nc_sess = nc_sess;
     }
+
+    /* (re)schedule the timer thread for rollback */
     if (ncc_commit_timeout_schedule(timeout)) {
         goto cleanup;
     }
 
+    if (timeout_changed) {
+        /* send notification about extending timeout for confirmed-commits */
+        np_send_notif_confirmed_commit(nc_sess, session, NP_CC_EXTEND, timeout, 0);
+    } else {
+        /* send notification about starting confirmed-commits */
+        np_send_notif_confirmed_commit(nc_sess, session, NP_CC_START, timeout, 0);
+    }
+
     /* sysrepo API */
     rc = sr_copy_config(user_sess->sess, NULL, SR_DS_CANDIDATE, np2srv.sr_timeout);
-    if ((rc == SR_ERR_LOCKED) && NP_IS_ORIG_NP(session)) {
+    if (rc == SR_ERR_LOCKED) {
         /* NETCONF error */
         sr_session_get_error(user_sess->sess, &err_info);
         np_err_sr2nc_in_use(session, err_info);
@@ -811,64 +938,64 @@ cleanup:
 
 int
 np2srv_rpc_commit_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const char *UNUSED(op_path),
-        const struct lyd_node *input, sr_event_t event, uint32_t UNUSED(request_id),
-        struct lyd_node *UNUSED(output), void *UNUSED(private_data))
+        const struct lyd_node *input, sr_event_t event, uint32_t UNUSED(request_id), struct lyd_node *UNUSED(output),
+        void *UNUSED(private_data))
 {
     int rc = SR_ERR_OK;
     struct np2_user_sess *user_sess = NULL;
+    struct nc_session *nc_sess;
     struct lyd_node *node;
     const sr_error_info_t *err_info;
-    const char *persist_id = NULL;
-    const char *persist;
+    const char *persist_id = NULL, *persist;
 
-    if (NP_IGNORE_RPC(session, event)) {
+    if (np_ignore_rpc(session, event, &rc)) {
         /* ignore in this case */
-        return SR_ERR_OK;
+        return rc;
     }
 
-    /* get the user session */
-    if ((rc = np_get_user_sess(session, NULL, &user_sess))) {
-        goto cleanup;
-    }
-
-    /* update sysrepo session datastore */
-    sr_session_switch_ds(user_sess->sess, SR_DS_RUNNING);
-
     /* LOCK */
     pthread_mutex_lock(&commit_ctx.lock);
+
     /* check if confirmed-commit */
-    lyd_find_path(input, "confirmed", 0, &node);
-    if (node) {
+    if (!lyd_find_path(input, "confirmed", 0, NULL)) {
         rc = np2srv_confirmed_commit_cb(session, input);
         goto cleanup;
     }
 
     /* persist-id */
-    lyd_find_path(input, "persist-id", 0, &node);
-    if (node) {
+    if (!lyd_find_path(input, "persist-id", 0, &node)) {
         persist_id = lyd_get_value(node);
     }
 
     persist = commit_ctx.persist;
     if ((persist && !persist_id) || (!persist && persist_id) || (persist && persist_id && strcmp(persist, persist_id))) {
-        np_err_invalid_value(session, "Confirming commit does not match pending confirmed commit.", persist_id);
+        np_err_invalid_value(session, "Commit does not match pending confirmed commit.", "persist_id");
         rc = SR_ERR_INVAL_ARG;
         goto cleanup;
     }
 
-    if (persist_id) {
-        /* confirming commit, set persist to NULL */
-        ncc_set_persist(NULL);
+    /* get the user session */
+    if ((rc = np_find_user_sess(session, __func__, &nc_sess, &user_sess))) {
+        goto cleanup;
     }
 
-    /* If there is a commit waiting to be confirmed, confirm it */
+    /* if there is a commit waiting to be confirmed, confirm it */
     if (commit_ctx.timer) {
+        if (!persist_id && (commit_ctx.nc_sess != nc_sess)) {
+            np_err_operation_failed(session, "Commit session does not match the pending confirmed commit session.");
+            rc = SR_ERR_OPERATION_FAILED;
+            goto cleanup;
+        }
         ncc_commit_confirmed();
+
+        /* send notification about complete confirmed-commits */
+        np_send_notif_confirmed_commit(nc_sess, session, NP_CC_COMPLETE, 0, 0);
     }
 
     /* sysrepo API */
+    sr_session_switch_ds(user_sess->sess, SR_DS_RUNNING);
     rc = sr_copy_config(user_sess->sess, NULL, SR_DS_CANDIDATE, np2srv.sr_timeout);
-    if ((rc == SR_ERR_LOCKED) && NP_IS_ORIG_NP(session)) {
+    if (rc == SR_ERR_LOCKED) {
         /* NETCONF error */
         sr_session_get_error(user_sess->sess, &err_info);
         np_err_sr2nc_in_use(session, err_info);
@@ -878,8 +1005,6 @@ np2srv_rpc_commit_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const c
         goto cleanup;
     }
 
-    /* success */
-
 cleanup:
     /* UNLOCK */
     pthread_mutex_unlock(&commit_ctx.lock);
@@ -889,35 +1014,78 @@ cleanup:
 
 int
 np2srv_rpc_cancel_commit_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const char *UNUSED(op_path),
-        const struct lyd_node *input, sr_event_t event, uint32_t UNUSED(request_id),
-        struct lyd_node *UNUSED(output), void *UNUSED(private_data))
+        const struct lyd_node *input, sr_event_t event, uint32_t UNUSED(request_id), struct lyd_node *UNUSED(output),
+        void *UNUSED(private_data))
 {
-    int rc = SR_ERR_OK;
+    int rc = SR_ERR_OK, is_locked;
+    struct nc_session *nc_sess;
     struct lyd_node *node;
     const char *persist_id = NULL, *persist = NULL;
+    uint32_t sr_id;
 
-    if (NP_IGNORE_RPC(session, event)) {
+    if (np_ignore_rpc(session, event, &rc)) {
         /* ignore in this case */
-        return SR_ERR_OK;
+        return rc;
+    }
+
+    /* get the NC session */
+    if ((rc = np_find_user_sess(session, __func__, &nc_sess, NULL))) {
+        goto cleanup;
     }
 
     /* persist-id */
-    lyd_find_path(input, "persist-id", 0, &node);
-    if (node) {
+    if (!lyd_find_path(input, "persist-id", 0, &node)) {
         persist_id = lyd_get_value(node);
     }
+
     /* LOCK */
     pthread_mutex_lock(&commit_ctx.lock);
-    persist = commit_ctx.persist;
-    if ((persist && !persist_id) || (!persist && persist_id) || (persist && persist_id && strcmp(persist, persist_id))) {
-        np_err_invalid_value(session, "Confirming commit does not match pending confirmed commit.", persist_id);
+
+    /* check there is a confirmed commit to cancel */
+    if (!commit_ctx.timer) {
+        np_err_invalid_value(session, "No pending confirmed commit to cancel.", NULL);
         rc = SR_ERR_INVAL_ARG;
         goto cleanup;
     }
 
-    ncc_commit_cancel();
+    /* check persist-id */
+    persist = commit_ctx.persist;
+    if ((persist && !persist_id) || (!persist && persist_id) || (persist && persist_id && strcmp(persist, persist_id))) {
+        np_err_invalid_value(session, "Cancel commit does not match pending confirmed commit.", persist_id);
+        rc = SR_ERR_INVAL_ARG;
+        goto cleanup;
+    }
 
-    /* success */
+    if (!persist_id) {
+        /* non-persist pending confirmed commit and session issuing the <cancel-commit> does not match
+         * the one issuing <commit> */
+        if (commit_ctx.nc_sess != nc_sess) {
+            np_err_operation_failed(session, "Cancel commit session does not match the pending confirmed commit session.");
+            rc = SR_ERR_OPERATION_FAILED;
+            goto cleanup;
+        }
+    } else {
+        /* make sure the running datastore in unlocked */
+        if ((rc = sr_get_lock(np2srv.sr_conn, SR_DS_RUNNING, NULL, &is_locked, &sr_id, NULL))) {
+            np_err_operation_failed(session, "Failed to learn the lock state of the <running> datastore.");
+            goto cleanup;
+        } else if (is_locked && (sr_session_get_id(((struct np2_user_sess *)nc_session_get_data(nc_sess))->sess) != sr_id)) {
+            np_err_in_use(session, sr_id);
+            rc = SR_ERR_LOCKED;
+            goto cleanup;
+        }
+
+        /* persist commit, set the NC session to use for the rollback */
+        assert(!commit_ctx.nc_sess);
+        commit_ctx.nc_sess = nc_sess;
+    }
+
+    /* rollback */
+    VRB("Performing confirmed commit rollback after receiving <cancel-commit>.");
+    ncc_changes_rollback_cb((union sigval)1);
+
+    /* send notification about canceling confirmed-commits */
+    np_send_notif_confirmed_commit(nc_sess, session, NP_CC_CANCEL, 0, 0);
 
 cleanup:
     /* UNLOCK */
