@@ -26,12 +26,13 @@
 
 #include <libyang/libyang.h>
 #include <sysrepo.h>
+#include <sysrepo/subscribed_notifications.h>
 
 #include "common.h"
 #include "compat.h"
 #include "config.h"
+#include "err_netconf.h"
 #include "log.h"
-#include "netconf_acm.h"
 
 /**
  * @brief Perform origin filtering.
@@ -100,20 +101,24 @@ np2srv_rpc_getdata_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const
         void *UNUSED(private_data))
 {
     struct lyd_node_term *leaf;
-    struct lyd_node *node, *select_data = NULL, *data = NULL;
-    struct np2_filter filter = {0};
+    struct lyd_node *node, *data = NULL;
+    char *xp_filter = NULL;
     int rc = SR_ERR_OK;
     struct np2_user_sess *user_sess = NULL;
     uint32_t i, max_depth = 0;
     struct ly_set *nodeset;
     sr_datastore_t ds;
     NC_WD_MODE nc_wd;
-    sr_get_oper_options_t get_opts = 0;
-    const char *username;
+    sr_get_options_t get_opts = 0;
 
-    if (NP_IGNORE_RPC(session, event)) {
+    if (np_ignore_rpc(session, event, &rc)) {
         /* ignore in this case */
-        return SR_ERR_OK;
+        return rc;
+    }
+
+    /* get the user session */
+    if ((rc = np_find_user_sess(session, __func__, NULL, &user_sess))) {
+        goto cleanup;
     }
 
     /* get default value for with-defaults */
@@ -129,9 +134,11 @@ np2srv_rpc_getdata_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const
         ds = SR_DS_CANDIDATE;
     } else if (!strcmp(leaf->value.ident->name, "operational")) {
         ds = SR_DS_OPERATIONAL;
+    } else if (!strcmp(leaf->value.ident->name, "factory-default")) {
+        ds = SR_DS_FACTORY_DEFAULT;
     } else {
         rc = SR_ERR_INVAL_ARG;
-        sr_session_set_error_message(session, "Datastore \"%s\" is not supported.", lyd_get_value(&leaf->node));
+        sr_session_set_error(session, NULL, rc, "Datastore \"%s\" is not supported.", lyd_get_value(&leaf->node));
         goto cleanup;
     }
 
@@ -140,30 +147,18 @@ np2srv_rpc_getdata_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const
     node = nodeset->count ? nodeset->dnodes[0] : NULL;
     ly_set_free(nodeset, NULL);
     if (node && !strcmp(node->schema->name, "subtree-filter")) {
-        if (op_filter_subtree2xpath(((struct lyd_node_any *)node)->value.tree, &filter)) {
-            rc = SR_ERR_INTERNAL;
-            goto cleanup;
+        if (((struct lyd_node_any *)node)->value.tree) {
+            if ((rc = srsn_filter_subtree2xpath(((struct lyd_node_any *)node)->value.tree, user_sess->sess, &xp_filter))) {
+                sr_session_dup_error(user_sess->sess, session);
+                goto cleanup;
+            }
         }
     } else {
-        filter.filters = malloc(sizeof *filter.filters);
-        if (!filter.filters) {
-            EMEM;
-            rc = SR_ERR_NO_MEMORY;
-            goto cleanup;
-        }
-        filter.count = 1;
-        filter.filters[0].str = node ? strdup(lyd_get_value(node)) : strdup("/*");
-        if (!filter.filters[0].str) {
-            EMEM;
-            rc = SR_ERR_NO_MEMORY;
-            goto cleanup;
-        }
-        filter.filters[0].selection = 1;
+        xp_filter = strdup(node ? lyd_get_value(node) : "/*");
     }
 
     /* config filter */
-    lyd_find_path(input, "config-filter", 0, &node);
-    if (node) {
+    if (!lyd_find_path(input, "config-filter", 0, &node)) {
         if (!strcmp(lyd_get_value(node), "false")) {
             get_opts |= SR_OPER_NO_CONFIG;
         } else {
@@ -172,20 +167,17 @@ np2srv_rpc_getdata_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const
     }
 
     /* depth */
-    lyd_find_path(input, "max-depth", 0, &node);
-    if (node && strcmp(lyd_get_value(node), "unbounded")) {
+    if (!lyd_find_path(input, "max-depth", 0, &node) && strcmp(lyd_get_value(node), "unbounded")) {
         max_depth = ((struct lyd_node_term *)node)->value.uint16;
     }
 
     /* origin */
-    lyd_find_path(input, "with-origin", 0, &node);
-    if (node) {
+    if (!lyd_find_path(input, "with-origin", 0, &node)) {
         get_opts |= SR_OPER_WITH_ORIGIN;
     }
 
     /* get with-defaults mode */
-    lyd_find_path(input, "with-defaults", 0, &node);
-    if (node) {
+    if (!lyd_find_path(input, "with-defaults", 0, &node)) {
         if (!strcmp(lyd_get_value(node), "report-all")) {
             nc_wd = NC_WD_ALL;
         } else if (!strcmp(lyd_get_value(node), "report-all-tagged")) {
@@ -198,21 +190,11 @@ np2srv_rpc_getdata_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const
         }
     }
 
-    /* get the user session */
-    if ((rc = np_get_user_sess(session, NULL, &user_sess))) {
-        goto cleanup;
-    }
-
     /* update sysrepo session datastore */
     sr_session_switch_ds(user_sess->sess, ds);
 
-    /*
-     * create the data tree for the data reply
-     */
-    if ((rc = op_filter_data_get(user_sess->sess, max_depth, get_opts, &filter, session, &select_data))) {
-        goto cleanup;
-    }
-    if ((rc = op_filter_data_filter(&select_data, &filter, 0, &data))) {
+    /* create the data tree for the data reply */
+    if ((rc = op_filter_data_get(user_sess->sess, max_depth, get_opts, xp_filter, session, &data))) {
         goto cleanup;
     }
 
@@ -224,21 +206,15 @@ np2srv_rpc_getdata_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const
     }
     ly_set_free(nodeset, NULL);
 
-    /* perform correct NACM filtering */
-    sr_session_get_orig_data(session, 1, NULL, (const void **)&username);
-    ncac_check_data_read_filter(&data, username);
-
     /* add output */
-    if (lyd_new_any(output, NULL, "data", data, 1, LYD_ANYDATA_DATATREE, 1, NULL)) {
+    if (lyd_new_any(output, NULL, "data", data, LYD_ANYDATA_DATATREE, LYD_NEW_ANY_USE_VALUE | LYD_NEW_VAL_OUTPUT, NULL)) {
+        rc = SR_ERR_LY;
         goto cleanup;
     }
     data = NULL;
 
-    /* success */
-
 cleanup:
-    op_filter_erase(&filter);
-    lyd_free_siblings(select_data);
+    free(xp_filter);
     lyd_free_siblings(data);
     np_release_user_sess(user_sess);
     return rc;
@@ -257,9 +233,9 @@ np2srv_rpc_editdata_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const
     const char *defop;
     int rc = SR_ERR_OK;
 
-    if (NP_IGNORE_RPC(session, event)) {
+    if (np_ignore_rpc(session, event, &rc)) {
         /* ignore in this case */
-        return SR_ERR_OK;
+        return rc;
     }
 
     /* get know which datastore is being affected */
@@ -272,7 +248,7 @@ np2srv_rpc_editdata_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const
         ds = SR_DS_CANDIDATE;
     } else {
         rc = SR_ERR_INVAL_ARG;
-        sr_session_set_error_message(session, "Datastore \"%s\" is not supported or writable.", lyd_get_value(&leaf->node));
+        sr_session_set_error(session, NULL, rc, "Datastore \"%s\" is not supported or writable.", lyd_get_value(&leaf->node));
         goto cleanup;
     }
 
@@ -292,19 +268,19 @@ np2srv_rpc_editdata_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const
     } else {
         assert(!strcmp(node->schema->name, "url"));
 #ifdef NP2SRV_URL_CAPAB
-        config = op_parse_url(lyd_get_value(node), LYD_PARSE_OPAQ | LYD_PARSE_ONLY, &rc, session);
+        config = op_parse_url(lyd_get_value(node), 0, &rc, session);
         if (rc) {
             goto cleanup;
         }
 #else
         rc = SR_ERR_UNSUPPORTED;
-        sr_session_set_error_message(session, "URL not supported.");
+        sr_session_set_error(session, NULL, rc, "URL not supported.");
         goto cleanup;
 #endif
     }
 
     /* get the user session */
-    if ((rc = np_get_user_sess(session, NULL, &user_sess))) {
+    if ((rc = np_find_user_sess(session, __func__, NULL, &user_sess))) {
         goto cleanup;
     }
 
@@ -312,20 +288,21 @@ np2srv_rpc_editdata_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const
     sr_session_switch_ds(user_sess->sess, ds);
 
     /* sysrepo API */
-    rc = sr_edit_batch(user_sess->sess, config, defop);
-    if (rc != SR_ERR_OK) {
+    if ((rc = sr_edit_batch(user_sess->sess, config, defop))) {
         goto cleanup;
     }
 
-    rc = sr_apply_changes(user_sess->sess, np2srv.sr_timeout);
-    if (rc != SR_ERR_OK) {
-        sr_session_dup_error(user_sess->sess, session);
+    if ((rc = sr_apply_changes(user_sess->sess, np2srv.sr_timeout))) {
+        /* specific edit-config error */
+        np_err_sr2nc_edit(session, user_sess->sess);
         goto cleanup;
     }
 
-    /* success */
-
 cleanup:
+    if (user_sess) {
+        /* discard any changes that possibly failed to be applied */
+        sr_discard_changes(user_sess->sess);
+    }
     lyd_free_siblings(config);
     np_release_user_sess(user_sess);
     return rc;
