@@ -18,6 +18,7 @@
 
 #include <setjmp.h>
 #include <stdarg.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -26,84 +27,57 @@
 #include <libyang/libyang.h>
 #include <nc_client.h>
 #include <sysrepo.h>
-#include "np_test.h"
-#include "np_test_config.h"
+#include <sysrepo/netconf_acm.h>
+
+#include "np2_test.h"
+#include "np2_test_config.h"
 
 static int
 local_setup(void **state)
 {
-    struct np_test *st;
-    sr_conn_ctx_t *conn;
     char test_name[256];
-    const char *module1 = NP_TEST_MODULE_DIR "/edit1.yang";
-    const char *module2 = NP_TEST_MODULE_DIR "/edit2.yang";
-    const char *module3 = NP_TEST_MODULE_DIR "/edit3.yang";
-    const char *module4 = NP_TEST_MODULE_DIR "/edit4.yang";
-    const char *module5 = NP_TEST_MODULE_DIR "/example1.yang";
-    const char *module6 = NP_TEST_MODULE_DIR "/example2.yang";
-    int rv;
+    const char *modules[] = {
+        NP_TEST_MODULE_DIR "/edit1.yang", NP_TEST_MODULE_DIR "/edit2.yang",
+        NP_TEST_MODULE_DIR "/edit3.yang", NP_TEST_MODULE_DIR "/edit4.yang", NP_TEST_MODULE_DIR "/example1.yang",
+        NP_TEST_MODULE_DIR "/example2.yang", NULL
+    };
+    int rc;
 
     /* get test name */
-    np_glob_setup_test_name(test_name);
+    np2_glob_test_setup_test_name(test_name);
 
-    /* Setup environment necessary for installing module */
-    rv = np_glob_setup_env(test_name);
-    assert_int_equal(rv, 0);
+    /* setup environment */
+    rc = np2_glob_test_setup_env(test_name);
+    assert_int_equal(rc, 0);
 
-    /* Connect to server and install test modules */
-    assert_int_equal(sr_connect(SR_CONN_DEFAULT, &conn), SR_ERR_OK);
-    assert_int_equal(sr_install_module(conn, module1, NULL, NULL), SR_ERR_OK);
-    assert_int_equal(sr_install_module(conn, module2, NULL, NULL), SR_ERR_OK);
-    assert_int_equal(sr_install_module(conn, module3, NULL, NULL), SR_ERR_OK);
-    assert_int_equal(sr_install_module(conn, module4, NULL, NULL), SR_ERR_OK);
-    assert_int_equal(sr_install_module(conn, module5, NULL, NULL), SR_ERR_OK);
-    assert_int_equal(sr_install_module(conn, module6, NULL, NULL), SR_ERR_OK);
-    assert_int_equal(sr_disconnect(conn), SR_ERR_OK);
+    /* setup netopeer2 server */
+    rc = np2_glob_test_setup_server(state, test_name, modules, 0);
+    assert_int_equal(rc, 0);
 
-    /* Setup netopeer2 server */
-    if (!(rv = np_glob_setup_np2(state, test_name))) {
-        st = *state;
-        /* Open the connection to start a session for the tests */
-        assert_int_equal(sr_connect(SR_CONN_DEFAULT, &st->conn), SR_ERR_OK);
-        assert_int_equal(sr_session_start(st->conn, SR_DS_RUNNING, &st->sr_sess), SR_ERR_OK);
-        assert_non_null(st->ctx = sr_get_context(st->conn));
-        rv |= setup_nacm(state);
-    }
-    return rv;
+    /* setup NACM */
+    rc = np2_glob_test_setup_nacm(state);
+    assert_int_equal(rc, 0);
+
+    return 0;
 }
 
 static int
 local_teardown(void **state)
 {
-    struct np_test *st = *state;
-    sr_conn_ctx_t *conn;
+    const char *modules[] = {"edit1", "edit2", "edit3", "edit4", "example1", "example2", NULL};
 
-    if (!st) {
+    if (!*state) {
         return 0;
     }
 
-    /* Close the session and connection needed for tests */
-    assert_int_equal(sr_session_stop(st->sr_sess), SR_ERR_OK);
-    assert_int_equal(sr_disconnect(st->conn), SR_ERR_OK);
-
-    /* Connect to server and remove test modules */
-    assert_int_equal(sr_connect(SR_CONN_DEFAULT, &conn), SR_ERR_OK);
-    assert_int_equal(sr_remove_module(conn, "edit1"), SR_ERR_OK);
-    assert_int_equal(sr_remove_module(conn, "edit2"), SR_ERR_OK);
-    assert_int_equal(sr_remove_module(conn, "edit3"), SR_ERR_OK);
-    assert_int_equal(sr_remove_module(conn, "edit4"), SR_ERR_OK);
-    assert_int_equal(sr_remove_module(conn, "example1"), SR_ERR_OK);
-    assert_int_equal(sr_remove_module(conn, "example2"), SR_ERR_OK);
-    assert_int_equal(sr_disconnect(conn), SR_ERR_OK);
-
-    /* Close netopeer2 server */
-    return np_glob_teardown(state);
+    /* close netopeer2 server */
+    return np2_glob_test_teardown(state, modules);
 }
 
 static int
 teardown_common(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data;
 
     data =
@@ -122,7 +96,7 @@ teardown_common(void **state)
 static void
 test_merge_edit1(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data;
 
     /* Send RPC editing module edit1 */
@@ -140,7 +114,7 @@ test_merge_edit1(void **state)
 static void
 test_merge_edit2(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data;
 
     /* Send RPC editing module edit2 */
@@ -162,7 +136,7 @@ test_merge_edit2(void **state)
 static void
 test_merge_edit2_fail(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data;
 
     /* Send invalid RPC editing module edit2 */
@@ -172,14 +146,14 @@ test_merge_edit2_fail(void **state)
             "  <num>ClearlyNotANumericValue</num>\n"
             "</top>\n";
     SEND_EDIT_RPC(st, data);
-    ASSERT_RPC_ERROR(st);
+    ASSERT_ERROR_REPLY(st);
     FREE_TEST_VARS(st);
 }
 
 static int
 setup_test_delete_edit1(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data;
 
     data = "<first xmlns=\"ed1\">TestFirst</first>\n";
@@ -192,7 +166,7 @@ setup_test_delete_edit1(void **state)
 static void
 test_delete_edit1(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data;
 
     /* Send rpc deleting config in module edit1 */
@@ -202,13 +176,13 @@ test_delete_edit1(void **state)
     FREE_TEST_VARS(st);
 
     /* Check if the config was deleted */
-    ASSERT_EMPTY_CONFIG(st);
+    ASSERT_EMPTY_CONFIG_FILTER(st, "/edit1:*");
 }
 
 static int
 setup_test_delete_edit2(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data;
 
     /* Send RPC editing module edit2 */
@@ -226,7 +200,7 @@ setup_test_delete_edit2(void **state)
 static void
 test_delete_edit2(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data;
 
     /* Send rpc deleting config in module edit2 */
@@ -236,26 +210,125 @@ test_delete_edit2(void **state)
     FREE_TEST_VARS(st);
 
     /* Check if the config was deleted */
-    ASSERT_EMPTY_CONFIG(st);
+    ASSERT_EMPTY_CONFIG_FILTER(st, "/edit2:*");
+}
+
+static int
+setup_test_delete_edit3(void **state)
+{
+    struct np2_test *st = *state;
+    const char *data;
+
+    /* send RPC editing module example1 */
+    data =
+            "<top xmlns=\"ex1\">\n"
+            "  <interface>\n"
+            "    <mtu>1024</mtu>\n"
+            "    <address>\n"
+            "      <name>10.0.0.1</name>\n"
+            "    </address>\n"
+            "  </interface>\n"
+            "</top>\n";
+    SEND_EDIT_RPC(st, data);
+    ASSERT_OK_REPLY(st);
+    FREE_TEST_VARS(st);
+    return 0;
+}
+
+static void
+test_delete_edit3(void **state)
+{
+    struct np2_test *st = *state;
+    const char *data;
+
+    /* send rpc deleting config in module edit2 with non-existing nodes */
+    data =
+            "<top xmlns=\"ex1\" xmlns:xc=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
+            "  <interface xc:operation=\"delete\">\n"
+            "    <name>my-name</name>\n"
+            "  </interface>\n"
+            "</top>\n";
+    SEND_EDIT_RPC(st, data);
+    ASSERT_ERROR_REPLY(st);
+    assert_string_equal(st->str,
+            "<rpc-error xmlns=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
+            "  <error-type>protocol</error-type>\n"
+            "  <error-tag>data-missing</error-tag>\n"
+            "  <error-severity>error</error-severity>\n"
+            "  <error-message xml:lang=\"en\">Node \"name\" to be deleted does not exist.</error-message>\n"
+            "</rpc-error>\n");
+    FREE_TEST_VARS(st);
+
+    data =
+            "<top xmlns=\"ex1\" xmlns:xc=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
+            "  <interface xc:operation=\"delete\">\n"
+            "    <mtu>2048</mtu>\n"
+            "    <address>\n"
+            "      <prefix-length>16</prefix-length>\n"
+            "    </address>\n"
+            "  </interface>\n"
+            "</top>\n";
+    SEND_EDIT_RPC(st, data);
+    ASSERT_ERROR_REPLY(st);
+    assert_string_equal(st->str,
+            "<rpc-error xmlns=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
+            "  <error-type>protocol</error-type>\n"
+            "  <error-tag>data-missing</error-tag>\n"
+            "  <error-severity>error</error-severity>\n"
+            "  <error-message xml:lang=\"en\">Node \"prefix-length\" to be deleted does not exist.</error-message>\n"
+            "</rpc-error>\n");
+    FREE_TEST_VARS(st);
+
+    data =
+            "<top xmlns=\"ex1\" xmlns:xc=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
+            "  <interface xc:operation=\"delete\">\n"
+            "    <mtu>2048</mtu>\n"
+            "    <address>\n"
+            "      <prefix-length xc:operation=\"merge\">16</prefix-length>\n"
+            "    </address>\n"
+            "  </interface>\n"
+            "</top>\n";
+    SEND_EDIT_RPC(st, data);
+    ASSERT_ERROR_REPLY(st);
+    assert_string_equal(st->str,
+            "<rpc-error xmlns=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
+            "  <error-type>application</error-type>\n"
+            "  <error-tag>operation-failed</error-tag>\n"
+            "  <error-severity>error</error-severity>\n"
+            "  <error-message xml:lang=\"en\">Invalid operation \"merge\" for node \"prefix-length\" with parent operation \"delete\".</error-message>\n"
+            "</rpc-error>\n");
+    FREE_TEST_VARS(st);
+
+    /* send valid rpc deleting config in module edit2 */
+    data =
+            "<top xmlns=\"ex1\" xmlns:xc=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
+            "  <interface xc:operation=\"delete\">\n"
+            "    <mtu>2048</mtu>\n"
+            "    <address/>\n"
+            "  </interface>\n"
+            "</top>\n";
+    SEND_EDIT_RPC(st, data);
+    ASSERT_OK_REPLY(st);
+    FREE_TEST_VARS(st);
 }
 
 static void
 test_delete_empty(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data;
 
     /* Try deleting a non-existent config, should fail */
     data = "<first xmlns=\"ed1\" xmlns:xc=\"urn:ietf:params:xml:ns:netconf:base:1.0\" xc:operation=\"delete\"/>\n";
     SEND_EDIT_RPC(st, data);
-    ASSERT_RPC_ERROR(st);
+    ASSERT_ERROR_REPLY(st);
     FREE_TEST_VARS(st);
 }
 
 static void
-test_merge_patrial(void **state)
+test_merge_partial(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *expected, *data =
             "<top xmlns=\"ed2\">\n"
             "  <name>TestSecond</name>\n"
@@ -267,7 +340,7 @@ test_merge_patrial(void **state)
     FREE_TEST_VARS(st);
 
     /* Check if merged successfully */
-    GET_CONFIG(st);
+    GET_CONFIG_FILTER(st, "/edit2:*");
     expected =
             "<get-config xmlns=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
             "  <data>\n"
@@ -283,7 +356,7 @@ test_merge_patrial(void **state)
 static int
 setup_test_merge_into_existing(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data =
             "<top xmlns=\"ed2\">\n"
             "  <name>TestSecond</name>\n"
@@ -298,7 +371,7 @@ setup_test_merge_into_existing(void **state)
 static void
 test_merge_into_existing(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *expected, *data =
             "<top xmlns=\"ed2\">\n"
             "  <num>123</num>\n"
@@ -309,7 +382,7 @@ test_merge_into_existing(void **state)
     FREE_TEST_VARS(st);
 
     /* Check if correctly merged */
-    GET_CONFIG(st);
+    GET_CONFIG_FILTER(st, "/edit2:*");
     expected =
             "<get-config xmlns=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
             "  <data>\n"
@@ -326,7 +399,7 @@ test_merge_into_existing(void **state)
 static int
 setup_test_merge_overwrite(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data =
             "<top xmlns=\"ed2\">\n"
             "  <name>TestSecond</name>\n"
@@ -342,7 +415,7 @@ setup_test_merge_overwrite(void **state)
 static void
 test_merge_overwrite(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *expected, *data =
             "<top xmlns=\"ed2\">\n"
             "  <num>456</num>\n"
@@ -353,7 +426,7 @@ test_merge_overwrite(void **state)
     FREE_TEST_VARS(st);
 
     /* Check if config was correctly overwritten */
-    GET_CONFIG(st);
+    GET_CONFIG_FILTER(st, "/edit2:*");
     expected =
             "<get-config xmlns=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
             "  <data>\n"
@@ -370,7 +443,7 @@ test_merge_overwrite(void **state)
 static int
 setup_test_replace(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data =
             "<top xmlns=\"ed3\" xmlns:xc=\"urn:ietf:params:xml:ns:netconf:base:1.0\" xc:operation=\"replace\">\n"
             "  <name>TestThird</name>\n"
@@ -386,7 +459,7 @@ setup_test_replace(void **state)
 static void
 test_replace(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *expected, *data =
             "<top xmlns=\"ed3\" xmlns:xc=\"urn:ietf:params:xml:ns:netconf:base:1.0\" xc:operation=\"replace\">\n"
             "  <name>TestThird</name>\n"
@@ -398,7 +471,7 @@ test_replace(void **state)
     FREE_TEST_VARS(st);
 
     /* Check if replaced correctly */
-    GET_CONFIG(st);
+    GET_CONFIG_FILTER(st, "/edit3:*");
     expected =
             "<get-config xmlns=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
             "  <data>\n"
@@ -415,7 +488,7 @@ test_replace(void **state)
 static void
 test_replace_create(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *expected, *data =
             "<top xmlns=\"ed3\" xmlns:xc=\"urn:ietf:params:xml:ns:netconf:base:1.0\" xc:operation=\"replace\">\n"
             "  <name>TestThird</name>\n"
@@ -427,7 +500,7 @@ test_replace_create(void **state)
     FREE_TEST_VARS(st);
 
     /* Check if created correctly */
-    GET_CONFIG(st);
+    GET_CONFIG_FILTER(st, "/edit3:*");
     expected =
             "<get-config xmlns=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
             "  <data>\n"
@@ -444,7 +517,7 @@ test_replace_create(void **state)
 static void
 test_create(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *expected, *data =
             "<first xmlns=\"ed1\" xmlns:xc=\"urn:ietf:params:xml:ns:netconf:base:1.0\" xc:operation=\"create\">"
             "TestFourth"
@@ -455,7 +528,7 @@ test_create(void **state)
     FREE_TEST_VARS(st);
 
     /* Check if config config now contains edit1 */
-    GET_CONFIG(st);
+    GET_CONFIG_FILTER(st, "/edit1:*");
     expected =
             "<get-config xmlns=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
             "  <data>\n"
@@ -469,7 +542,7 @@ test_create(void **state)
 static int
 setup_test_create_fail(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data =
             "<first xmlns=\"ed1\" xmlns:xc=\"urn:ietf:params:xml:ns:netconf:base:1.0\" xc:operation=\"create\">"
             "TestFourth"
@@ -484,21 +557,21 @@ setup_test_create_fail(void **state)
 static void
 test_create_fail(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data =
             "<first xmlns=\"ed1\" xmlns:xc=\"urn:ietf:params:xml:ns:netconf:base:1.0\" xc:operation=\"create\">"
             "TestFourth"
             "</first>\n";
 
     SEND_EDIT_RPC(st, data);
-    ASSERT_RPC_ERROR(st);
+    ASSERT_ERROR_REPLY(st);
     FREE_TEST_VARS(st);
 }
 
 static int
 setup_test_remove(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data = "<first xmlns=\"ed1\">TestFirst</first>\n";
 
     SEND_EDIT_RPC(st, data);
@@ -510,31 +583,31 @@ setup_test_remove(void **state)
 static void
 test_remove(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data = "<first xmlns=\"ed1\" xmlns:xc=\"urn:ietf:params:xml:ns:netconf:base:1.0\" xc:operation=\"remove\"></first>\n";
 
     SEND_EDIT_RPC(st, data);
     ASSERT_OK_REPLY(st);
     FREE_TEST_VARS(st);
-    ASSERT_EMPTY_CONFIG(st);
+    ASSERT_EMPTY_CONFIG_FILTER(st, "/edit1:*");
 }
 
 static void
 test_remove_empty(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data = "<first xmlns=\"ed1\" xmlns:xc=\"urn:ietf:params:xml:ns:netconf:base:1.0\" xc:operation=\"remove\"></first>\n";
 
     SEND_EDIT_RPC(st, data);
     ASSERT_OK_REPLY(st);
     FREE_TEST_VARS(st);
-    ASSERT_EMPTY_CONFIG(st);
+    ASSERT_EMPTY_CONFIG_FILTER(st, "/edit1:*");
 }
 
 static int
 setup_test_ex1(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data =
             "<top xmlns=\"ex1\">\n"
             "  <interface>\n"
@@ -553,7 +626,7 @@ static void
 test_ex1(void **state)
 {
     /* First example for edit-config from rfc 6241 section 7.2 */
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *expected, *data;
 
     data =
@@ -572,7 +645,7 @@ test_ex1(void **state)
     ASSERT_OK_REPLY(st);
     FREE_TEST_VARS(st);
 
-    GET_CONFIG(st);
+    GET_CONFIG_FILTER(st, "/example1:*");
     expected =
             "<get-config xmlns=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
             "  <data>\n"
@@ -595,7 +668,7 @@ test_ex1(void **state)
 static int
 setup_test_ex2(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data =
             "<top xmlns=\"ex2\">\n"
             "  <protocols>\n"
@@ -625,7 +698,7 @@ static void
 test_ex2(void **state)
 {
     /* Second example for edit-config from rfc 6241 section 7.2 */
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *expected, *data =
             "<top xmlns=\"ex2\">\n"
             "  <protocols>\n"
@@ -646,7 +719,7 @@ test_ex2(void **state)
     ASSERT_OK_REPLY(st);
     FREE_TEST_VARS(st);
 
-    GET_CONFIG(st);
+    GET_CONFIG_FILTER(st, "/example2:*");
     expected =
             "<get-config xmlns=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
             "  <data>\n"
@@ -673,7 +746,7 @@ test_ex2(void **state)
 static void
 test_autodel_case(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data;
 
     /* create case #1 */
@@ -687,7 +760,7 @@ test_autodel_case(void **state)
     FREE_TEST_VARS(st);
 
     /* check data */
-    GET_CONFIG(st);
+    GET_CONFIG_FILTER(st, "/edit4:*");
     assert_non_null(strstr(st->str, "l1"));
     assert_non_null(strstr(st->str, "l2"));
     FREE_TEST_VARS(st);
@@ -702,7 +775,7 @@ test_autodel_case(void **state)
     FREE_TEST_VARS(st);
 
     /* check data */
-    GET_CONFIG(st);
+    GET_CONFIG_FILTER(st, "/edit4:*");
     assert_null(strstr(st->str, "l1"));
     assert_null(strstr(st->str, "l2"));
     assert_non_null(strstr(st->str, "c2"));
@@ -720,7 +793,7 @@ test_autodel_case(void **state)
     FREE_TEST_VARS(st);
 
     /* check data */
-    GET_CONFIG(st);
+    GET_CONFIG_FILTER(st, "/edit4:*");
     assert_null(strstr(st->str, "c2"));
     assert_non_null(strstr(st->str, "l3"));
     FREE_TEST_VARS(st);
@@ -736,24 +809,43 @@ test_autodel_case(void **state)
     FREE_TEST_VARS(st);
 
     /* check data */
-    GET_CONFIG(st);
+    GET_CONFIG_FILTER(st, "/edit4:*");
     assert_null(strstr(st->str, "l3"));
     assert_non_null(strstr(st->str, "l4"));
     assert_non_null(strstr(st->str, "l5"));
     FREE_TEST_VARS(st);
 }
 
+static void
+test_unknown(void **state)
+{
+    struct np2_test *st = *state;
+    const char *data;
+
+    /* send unknown data, should fail */
+    data = "<first xmlns=\"invalid\"/>\n";
+    SEND_EDIT_RPC(st, data);
+    ASSERT_ERROR_REPLY(st);
+    FREE_TEST_VARS(st);
+}
+
 int
 main(int argc, char **argv)
 {
+    if (np2_is_nacm_recovery()) {
+        puts("Running as NACM_RECOVERY_USER. Tests will not run correctly as this user bypases NACM. Skipping.");
+        return 0;
+    }
+
     const struct CMUnitTest tests[] = {
         cmocka_unit_test_teardown(test_merge_edit1, teardown_common),
         cmocka_unit_test_teardown(test_merge_edit2, teardown_common),
         cmocka_unit_test_teardown(test_merge_edit2_fail, teardown_common),
         cmocka_unit_test_setup(test_delete_edit1, setup_test_delete_edit1),
         cmocka_unit_test_setup(test_delete_edit2, setup_test_delete_edit2),
+        cmocka_unit_test_setup_teardown(test_delete_edit3, setup_test_delete_edit3, teardown_common),
         cmocka_unit_test(test_delete_empty),
-        cmocka_unit_test_teardown(test_merge_patrial, teardown_common),
+        cmocka_unit_test_teardown(test_merge_partial, teardown_common),
         cmocka_unit_test_setup_teardown(test_merge_into_existing, setup_test_merge_into_existing, teardown_common),
         cmocka_unit_test_setup_teardown(test_merge_overwrite, setup_test_merge_overwrite, teardown_common),
         cmocka_unit_test_setup_teardown(test_replace, setup_test_replace, teardown_common),
@@ -765,6 +857,7 @@ main(int argc, char **argv)
         cmocka_unit_test_setup_teardown(test_ex1, setup_test_ex1, teardown_common),
         cmocka_unit_test_setup_teardown(test_ex2, setup_test_ex2, teardown_common),
         cmocka_unit_test_teardown(test_autodel_case, teardown_common),
+        cmocka_unit_test(test_unknown),
     };
 
     nc_verbosity(NC_VERB_WARNING);
