@@ -18,6 +18,7 @@
 
 #include <setjmp.h>
 #include <stdarg.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
@@ -27,83 +28,65 @@
 #include <libyang/libyang.h>
 #include <nc_client.h>
 #include <sysrepo.h>
+#include <sysrepo/netconf_acm.h>
 
-#include "np_test.h"
-#include "np_test_config.h"
+#include "np2_test.h"
+#include "np2_test_config.h"
 
 static int
 local_setup(void **state)
 {
-    struct np_test *st;
-    sr_conn_ctx_t *conn;
+    struct np2_test *st;
     char test_name[256];
-    const char *module1 = NP_TEST_MODULE_DIR "/edit1.yang";
-    const char *module2 = NP_TEST_MODULE_DIR "/example2.yang";
-    const char *module3 = NP_TEST_MODULE_DIR "/nacm-test1.yang";
-    const char *module4 = NP_TEST_MODULE_DIR "/nacm-test2.yang";
-    int rv;
+    const char *modules[] = {
+        NP_TEST_MODULE_DIR "/edit1.yang", NP_TEST_MODULE_DIR "/example2.yang",
+        NP_TEST_MODULE_DIR "/nacm-test1.yang", NP_TEST_MODULE_DIR "/nacm-test2.yang", NULL
+    };
+    int rc;
 
     /* get test name */
-    np_glob_setup_test_name(test_name);
+    np2_glob_test_setup_test_name(test_name);
 
-    /* setup environment necessary for installing module */
-    rv = np_glob_setup_env(test_name);
-    assert_int_equal(rv, 0);
-
-    /* connect to server and install test modules */
-    assert_int_equal(sr_connect(SR_CONN_DEFAULT, &conn), SR_ERR_OK);
-    assert_int_equal(sr_install_module(conn, module1, NULL, NULL), SR_ERR_OK);
-    assert_int_equal(sr_install_module(conn, module2, NULL, NULL), SR_ERR_OK);
-    assert_int_equal(sr_install_module(conn, module3, NULL, NULL), SR_ERR_OK);
-    assert_int_equal(sr_install_module(conn, module4, NULL, NULL), SR_ERR_OK);
-    assert_int_equal(sr_disconnect(conn), SR_ERR_OK);
+    /* setup environment */
+    rc = np2_glob_test_setup_env(test_name);
+    assert_int_equal(rc, 0);
 
     /* setup netopeer2 server */
-    if (!(rv = np_glob_setup_np2(state, test_name))) {
-        st = *state;
-        /* Open two connections to start a session for the tests
-         * One for Candidate and other for running
-         */
-        assert_int_equal(sr_connect(SR_CONN_DEFAULT, &st->conn), SR_ERR_OK);
-        assert_int_equal(sr_session_start(st->conn, SR_DS_RUNNING, &st->sr_sess), SR_ERR_OK);
-        assert_non_null(st->ctx = sr_get_context(st->conn));
-        assert_int_equal(sr_session_start(st->conn, SR_DS_CANDIDATE, &st->sr_sess2), SR_ERR_OK);
-        rv |= setup_nacm(state);
-    }
-    return rv;
+    rc = np2_glob_test_setup_server(state, test_name, modules, 0);
+    assert_int_equal(rc, 0);
+    st = *state;
+
+    /* candidate session */
+    assert_int_equal(sr_session_start(st->conn, SR_DS_CANDIDATE, &st->sr_sess2), SR_ERR_OK);
+
+    /* setup NACM */
+    rc = np2_glob_test_setup_nacm(state);
+    assert_int_equal(rc, 0);
+
+    return 0;
 }
 
 static int
 local_teardown(void **state)
 {
-    struct np_test *st = *state;
-    sr_conn_ctx_t *conn;
+    struct np2_test *st = *state;
+    const char *modules[] = {"edit1", "example2", "nacm-test1", "nacm-test2", NULL};
 
     if (!st) {
         return 0;
     }
 
-    /* Close the sessions and connection needed for tests */
-    assert_int_equal(sr_session_stop(st->sr_sess), SR_ERR_OK);
+    /* close the session */
     assert_int_equal(sr_session_stop(st->sr_sess2), SR_ERR_OK);
-    assert_int_equal(sr_disconnect(st->conn), SR_ERR_OK);
-
-    /* connect to server and remove test modules */
-    assert_int_equal(sr_connect(SR_CONN_DEFAULT, &conn), SR_ERR_OK);
-    assert_int_equal(sr_remove_module(conn, "edit1"), SR_ERR_OK);
-    assert_int_equal(sr_remove_module(conn, "example2"), SR_ERR_OK);
-    assert_int_equal(sr_remove_module(conn, "nacm-test1"), SR_ERR_OK);
-    assert_int_equal(sr_remove_module(conn, "nacm-test2"), SR_ERR_OK);
-    assert_int_equal(sr_disconnect(conn), SR_ERR_OK);
 
     /* close netopeer2 server */
-    return np_glob_teardown(state);
+    return np2_glob_test_teardown(state, modules);
 }
 
 static int
 teardown_common(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data =
             "<first xmlns=\"ed1\" xmlns:xc=\"urn:ietf:params:xml:ns:netconf:base:1.0\" "
             "xc:operation=\"remove\"></first>\n"
@@ -142,7 +125,7 @@ teardown_common(void **state)
 static int
 setup_test_exec_get(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data =
             "<nacm xmlns=\"urn:ietf:params:xml:ns:yang:ietf-netconf-acm\">\n"
             "  <write-default>permit</write-default>\n"
@@ -174,7 +157,7 @@ setup_test_exec_get(void **state)
 static void
 test_exec_get(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
 
     /* get and get-config bypass execution permissions */
     GET_FILTER(st, NULL);
@@ -189,7 +172,7 @@ test_exec_get(void **state)
 static int
 setup_test_read_default(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data =
             "<nacm xmlns=\"urn:ietf:params:xml:ns:yang:ietf-netconf-acm\">\n"
             "  <read-default>deny</read-default>\n"
@@ -203,7 +186,7 @@ setup_test_read_default(void **state)
 static void
 test_read_default(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *expected =
             "<get xmlns=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
             "  <data/>\n"
@@ -218,7 +201,7 @@ test_read_default(void **state)
 static int
 setup_test_read_default_allow_path(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data =
             "<first xmlns=\"ed1\">TestFirst</first>\n"
             "<nacm xmlns=\"urn:ietf:params:xml:ns:yang:ietf-netconf-acm\">\n"
@@ -244,7 +227,7 @@ setup_test_read_default_allow_path(void **state)
 static void
 test_read_default_allow_path(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *expected =
             "<get xmlns=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
             "  <data>\n"
@@ -261,7 +244,7 @@ test_read_default_allow_path(void **state)
 static int
 setup_test_get_config(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data =
             "<first xmlns=\"ed1\">TestFirst</first>\n"
             "<nacm xmlns=\"urn:ietf:params:xml:ns:yang:ietf-netconf-acm\">\n"
@@ -283,26 +266,10 @@ setup_test_get_config(void **state)
     return 0;
 }
 
-static void
-test_get_config(void **state)
-{
-    struct np_test *st = *state;
-    const char *expected;
-
-    /* Since reading of this node is denied it should return empty config */
-    GET_CONFIG(st);
-    expected =
-            "<get-config xmlns=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
-            "  <data/>\n"
-            "</get-config>\n";
-    assert_string_equal(st->str, expected);
-    FREE_TEST_VARS(st);
-}
-
 static void
 test_get_config_filter(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *expected;
 
     /* Using a filter on a denied node should not cause access denied error */
@@ -318,7 +285,7 @@ test_get_config_filter(void **state)
 static int
 setup_test_xpath_filter_denied(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data =
             "<people xmlns=\"urn:nt2\">\n"
             "  <name>John</name>\n"
@@ -355,7 +322,7 @@ static void
 test_xpath_filter_denied(void **state)
 {
     /* Issue #846 */
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *expected, *filter = "/people[weight>100]";
 
     GET_CONFIG_FILTER(st, filter);
@@ -374,7 +341,7 @@ test_xpath_filter_denied(void **state)
 static int
 setup_test_filter_key_list(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data =
             "<people xmlns=\"urn:nt2\">\n"
             "  <name>John</name>\n"
@@ -411,14 +378,17 @@ static void
 test_filter_key_list(void **state)
 {
     /* Issue #755 */
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *expected;
 
-    GET_CONFIG(st);
+    GET_CONFIG_FILTER(st, "/nacm-test2:*");
     expected =
             "<get-config xmlns=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
             "  <data>\n"
             "    <people xmlns=\"urn:nt2\">\n"
+            "      <name>Arnold</name>\n"
+            "    </people>\n"
+            "    <people xmlns=\"urn:nt2\">\n"
             "      <name>John</name>\n"
             "      <weight>75</weight>\n"
             "    </people>\n"
@@ -426,9 +396,6 @@ test_filter_key_list(void **state)
             "      <name>Thomas</name>\n"
             "      <weight>100</weight>\n"
             "    </people>\n"
-            "    <people xmlns=\"urn:nt2\">\n"
-            "      <name>Arnold</name>\n"
-            "    </people>\n"
             "  </data>\n"
             "</get-config>\n";
     assert_string_equal(st->str, expected);
@@ -438,7 +405,7 @@ test_filter_key_list(void **state)
 static int
 setup_test_rule_wildcard_groups(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data =
             "<first xmlns=\"ed1\">TestFirst</first>\n"
             "<nacm xmlns=\"urn:ietf:params:xml:ns:yang:ietf-netconf-acm\">\n"
@@ -465,7 +432,7 @@ static void
 test_rule_wildcard_groups(void **state)
 {
     /* Issue #619 */
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *expected =
             "<get xmlns=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
             "  <data>\n"
@@ -482,7 +449,7 @@ test_rule_wildcard_groups(void **state)
 static int
 setup_edit_config(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data =
             "<nacm xmlns=\"urn:ietf:params:xml:ns:yang:ietf-netconf-acm\">\n"
             "  <write-default>deny</write-default>\n"
@@ -497,7 +464,7 @@ setup_edit_config(void **state)
 static int
 teardown_edit_config(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data =
             "<nacm xmlns=\"urn:ietf:params:xml:ns:yang:ietf-netconf-acm\">\n"
             "  <write-default>permit</write-default>\n"
@@ -512,11 +479,11 @@ teardown_edit_config(void **state)
 static void
 test_edit_config(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data = "<first xmlns=\"ed1\">TestFirst</first>\n";
 
     SEND_EDIT_RPC(st, data);
-    ASSERT_RPC_ERROR(st);
+    ASSERT_ERROR_REPLY(st);
     /* Check if correct error-tag */
     assert_string_equal(lyd_get_value(lyd_child(lyd_child(st->envp))->next), "access-denied");
     FREE_TEST_VARS(st);
@@ -525,7 +492,7 @@ test_edit_config(void **state)
 static int
 setup_test_edit_config_permit(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data =
             "<first xmlns=\"ed1\">TestFirst</first>\n"
             "<nacm xmlns=\"urn:ietf:params:xml:ns:yang:ietf-netconf-acm\">\n"
@@ -551,7 +518,7 @@ setup_test_edit_config_permit(void **state)
 static void
 test_edit_config_permit(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data = "<first xmlns=\"ed1\">TestFirst</first>\n";
 
     SEND_EDIT_RPC(st, data);
@@ -562,7 +529,7 @@ test_edit_config_permit(void **state)
 static int
 setup_test_edit_config_update(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data =
             "<nacm xmlns=\"urn:ietf:params:xml:ns:yang:ietf-netconf-acm\">\n"
             "  <write-default>deny</write-default>\n"
@@ -587,12 +554,12 @@ setup_test_edit_config_update(void **state)
 static void
 test_edit_config_update(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *expected, *data = "<first xmlns=\"ed1\">Test</first>\n";
 
     /* Creating is not permited */
     SEND_EDIT_RPC(st, data);
-    ASSERT_RPC_ERROR(st);
+    ASSERT_ERROR_REPLY(st);
     /* Check if correct error-tag */
     assert_string_equal(lyd_get_value(lyd_child(lyd_child(st->envp))->next), "access-denied");
     FREE_TEST_VARS(st);
@@ -607,7 +574,7 @@ test_edit_config_update(void **state)
     ASSERT_OK_REPLY(st);
     FREE_TEST_VARS(st);
 
-    GET_CONFIG(st);
+    GET_CONFIG_FILTER(st, "/edit1:*");
     expected =
             "<get-config xmlns=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
             "  <data>\n"
@@ -621,7 +588,7 @@ test_edit_config_update(void **state)
 static int
 setup_test_edit_config_subtree(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data =
             "<top xmlns=\"ex2\">\n"
             "  <protocols>\n"
@@ -663,7 +630,7 @@ setup_test_edit_config_subtree(void **state)
 static void
 test_edit_config_subtree(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data =
             "<top xmlns=\"ex2\">\n"
             "  <protocols>\n"
@@ -698,7 +665,7 @@ test_edit_config_subtree(void **state)
 
     /* Adding another area should fail */
     SEND_EDIT_RPC(st, data);
-    ASSERT_RPC_ERROR(st);
+    ASSERT_ERROR_REPLY(st);
     /* Check if correct error-tag */
     assert_string_equal(lyd_get_value(lyd_child(lyd_child(st->envp))->next), "access-denied");
     FREE_TEST_VARS(st);
@@ -707,9 +674,8 @@ test_edit_config_subtree(void **state)
             "<top xmlns=\"ex2\">\n"
             "  <protocols>\n"
             "    <ospf>\n"
-            "      <area>\n"
-            "        <name xmlns:pref=\"urn:ietf:params:xml:ns:netconf:base:1.0\""
-            "pref:operation=\"remove\">0.0.0.0</name>\n"
+            "      <area xmlns:pref=\"urn:ietf:params:xml:ns:netconf:base:1.0\" pref:operation=\"remove\">\n"
+            "        <name>0.0.0.0</name>\n"
             "      </area>\n"
             "    </ospf>\n"
             "  </protocols>\n"
@@ -724,7 +690,7 @@ test_edit_config_subtree(void **state)
 static int
 setup_test_edit_config_when(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data =
             "<top xmlns=\"urn:nt1\">\n"
             "  <name>Test</name>\n"
@@ -758,7 +724,7 @@ setup_test_edit_config_when(void **state)
 static void
 test_edit_config_when(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *expected, *data =
             "<top xmlns=\"urn:nt1\">\n"
             "  <num xmlns:p=\"urn:ietf:params:xml:ns:netconf:base:1.0\" p:operation=\"remove\"/>\n"
@@ -766,7 +732,7 @@ test_edit_config_when(void **state)
 
     /* Removing the num should fail */
     SEND_EDIT_RPC(st, data);
-    ASSERT_RPC_ERROR(st);
+    ASSERT_ERROR_REPLY(st);
     /* Check if correct error-tag */
     assert_string_equal(lyd_get_value(lyd_child(lyd_child(st->envp))->next), "access-denied");
     FREE_TEST_VARS(st);
@@ -780,7 +746,7 @@ test_edit_config_when(void **state)
     SEND_EDIT_RPC(st, data);
     ASSERT_OK_REPLY(st);
     FREE_TEST_VARS(st);
-    GET_CONFIG(st);
+    GET_CONFIG_FILTER(st, "/nacm-test1:*");
     expected = "<get-config xmlns=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n  <data/>\n</get-config>\n";
     assert_string_equal(st->str, expected);
     FREE_TEST_VARS(st);
@@ -789,7 +755,7 @@ test_edit_config_when(void **state)
 static void
 test_copy_config_running2startup(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
 
     /* From running to startup only needs execute on copy-config */
     st->rpc = nc_rpc_copy(NC_DATASTORE_STARTUP, NULL, NC_DATASTORE_RUNNING, NULL, NC_WD_ALL, NC_PARAMTYPE_CONST);
@@ -802,7 +768,7 @@ test_copy_config_running2startup(void **state)
 static int
 setup_test_copy_config_ds2ds_fail_read(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data =
             "<nacm xmlns=\"urn:ietf:params:xml:ns:yang:ietf-netconf-acm\">\n"
             "  <write-default>deny</write-default>\n"
@@ -832,7 +798,7 @@ setup_test_copy_config_ds2ds_fail_read(void **state)
 static int
 setup_test_copy_config_ds2ds_fail_write(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data =
             "<nacm xmlns=\"urn:ietf:params:xml:ns:yang:ietf-netconf-acm\">\n"
             "  <write-default>deny</write-default>\n"
@@ -862,12 +828,12 @@ setup_test_copy_config_ds2ds_fail_write(void **state)
 static void
 test_copy_config_ds2ds_fail_read(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
 
     st->rpc = nc_rpc_copy(NC_DATASTORE_RUNNING, NULL, NC_DATASTORE_CANDIDATE, NULL, NC_WD_ALL, NC_PARAMTYPE_CONST);
     st->msgtype = nc_send_rpc(st->nc_sess, st->rpc, 1000, &st->msgid);
     assert_int_equal(NC_MSG_RPC, st->msgtype);
-    ASSERT_RPC_ERROR(st);
+    ASSERT_ERROR_REPLY(st);
     /* Check if correct error-tag */
     assert_string_equal(lyd_get_value(lyd_child(lyd_child(st->envp))->next), "access-denied");
     FREE_TEST_VARS(st);
@@ -882,14 +848,14 @@ test_copy_config_ds2ds_fail_write(void **state)
 static void
 test_delete_config(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data;
 
     /* Should be disabled by default */
     st->rpc = nc_rpc_delete(NC_DATASTORE_STARTUP, NULL, NC_PARAMTYPE_CONST);
     st->msgtype = nc_send_rpc(st->nc_sess, st->rpc, 1000, &st->msgid);
     assert_int_equal(NC_MSG_RPC, st->msgtype);
-    ASSERT_RPC_ERROR(st);
+    ASSERT_ERROR_REPLY(st);
     assert_string_equal(lyd_get_value(lyd_child(lyd_child(st->envp))->next), "access-denied");
     FREE_TEST_VARS(st);
 
@@ -928,7 +894,7 @@ test_delete_config(void **state)
 static int
 setup_test_commit(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     char *data =
             "<nacm xmlns=\"urn:ietf:params:xml:ns:yang:ietf-netconf-acm\">\n"
             "  <rule-list>\n"
@@ -966,14 +932,14 @@ setup_test_commit(void **state)
 static void
 test_commit(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data, *expected;
 
     /* Should fail since candidate has element that is denied to merge */
     st->rpc = nc_rpc_commit(0, 0, NULL, NULL, NC_PARAMTYPE_CONST);
     st->msgtype = nc_send_rpc(st->nc_sess, st->rpc, 1000, &st->msgid);
     assert_int_equal(NC_MSG_RPC, st->msgtype);
-    ASSERT_RPC_ERROR(st);
+    ASSERT_ERROR_REPLY(st);
     assert_string_equal(lyd_get_value(lyd_child(lyd_child(st->envp))->next), "access-denied");
     FREE_TEST_VARS(st);
 
@@ -992,7 +958,7 @@ test_commit(void **state)
     ASSERT_OK_REPLY(st);
     FREE_TEST_VARS(st);
 
-    GET_CONFIG(st);
+    GET_CONFIG_FILTER(st, "/nacm-test1:*");
     expected =
             "<get-config xmlns=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
             "  <data>\n"
@@ -1009,7 +975,7 @@ test_commit(void **state)
 static int
 setup_test_discard_changes(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     char *data =
             "<nacm xmlns=\"urn:ietf:params:xml:ns:yang:ietf-netconf-acm\">\n"
             "  <rule-list>\n"
@@ -1038,7 +1004,7 @@ setup_test_discard_changes(void **state)
 static void
 test_discard_changes(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *expected;
 
     /* Should succeed despite not having rights over the datastore nodes */
@@ -1048,7 +1014,7 @@ test_discard_changes(void **state)
     ASSERT_OK_REPLY(st);
     FREE_TEST_VARS(st);
 
-    GET_CONFIG_DS_FILTER(st, NC_DATASTORE_CANDIDATE, NULL);
+    GET_CONFIG_DS_FILTER(st, NC_DATASTORE_CANDIDATE, "/edit1:*");
     expected =
             "<get-config xmlns=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
             "  <data/>\n"
@@ -1060,14 +1026,14 @@ test_discard_changes(void **state)
 static void
 test_kill_session(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data;
 
     /* Should fail since the rpc is disallowed */
     st->rpc = nc_rpc_kill(nc_session_get_id(st->nc_sess2));
     st->msgtype = nc_send_rpc(st->nc_sess, st->rpc, 1000, &st->msgid);
     assert_int_equal(NC_MSG_RPC, st->msgtype);
-    ASSERT_RPC_ERROR(st);
+    ASSERT_ERROR_REPLY(st);
     /* Check if correct error-tag */
     assert_string_equal(lyd_get_value(lyd_child(lyd_child(st->envp))->next), "access-denied");
     FREE_TEST_VARS(st);
@@ -1099,8 +1065,8 @@ test_kill_session(void **state)
 int
 main(int argc, char **argv)
 {
-    if (is_nacm_rec_uid()) {
-        puts("Running as NACM_RECOVERY_UID. Tests will not run correctly as this user bypases NACM. Skipping.");
+    if (np2_is_nacm_recovery()) {
+        puts("Running as NACM_RECOVERY_USER. Tests will not run correctly as this user bypases NACM. Skipping.");
         return 0;
     }
 
@@ -1121,9 +1087,6 @@ main(int argc, char **argv)
             cmocka_unit_test_setup_teardown(test_read_default_allow_path,
                     setup_test_read_default_allow_path,
                     teardown_common),
-            cmocka_unit_test_setup_teardown(test_get_config,
-                    setup_test_get_config,
-                    teardown_common),
             cmocka_unit_test_setup_teardown(test_get_config_filter,
                     setup_test_get_config,
                     teardown_common),
@@ -1137,6 +1100,7 @@ main(int argc, char **argv)
                     setup_test_rule_wildcard_groups,
                     teardown_common),
         };
+
         return cmocka_run_group_tests(tests, local_setup, local_teardown);
     } else {
         /* sysrepo super run with write tests */
@@ -1150,9 +1114,6 @@ main(int argc, char **argv)
             cmocka_unit_test_setup_teardown(test_read_default_allow_path,
                     setup_test_read_default_allow_path,
                     teardown_common),
-            cmocka_unit_test_setup_teardown(test_get_config,
-                    setup_test_get_config,
-                    teardown_common),
             cmocka_unit_test_setup_teardown(test_get_config_filter,
                     setup_test_get_config,
                     teardown_common),
@@ -1198,6 +1159,7 @@ main(int argc, char **argv)
             cmocka_unit_test_teardown(test_kill_session,
                     teardown_common),
         };
+
         return cmocka_run_group_tests(tests, local_setup, local_teardown);
     }
 }
