@@ -18,8 +18,10 @@
 
 #include <setjmp.h>
 #include <stdarg.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/wait.h>
 #include <time.h>
 #include <unistd.h>
 
@@ -27,74 +29,57 @@
 #include <libyang/libyang.h>
 #include <nc_client.h>
 #include <sysrepo.h>
+#include <sysrepo/netconf_acm.h>
 
-#include "np_test.h"
-#include "np_test_config.h"
+#include "np2_test.h"
+#include "np2_test_config.h"
 
 static int
 local_setup(void **state)
 {
-    struct np_test *st;
-    sr_conn_ctx_t *conn;
+    struct np2_test *st;
+    const char *modules[] = {NP_TEST_MODULE_DIR "/notif1.yang", NP_TEST_MODULE_DIR "/notif2.yang", NULL};
     char test_name[256];
-    const char *module1 = NP_TEST_MODULE_DIR "/notif1.yang";
-    const char *module2 = NP_TEST_MODULE_DIR "/notif2.yang";
-    int rv;
+    int rc;
 
     /* get test name */
-    np_glob_setup_test_name(test_name);
+    np2_glob_test_setup_test_name(test_name);
 
-    /* setup environment necessary for installing module */
-    rv = np_glob_setup_env(test_name);
-    assert_int_equal(rv, 0);
-
-    /* connect to server and install test modules */
-    assert_int_equal(sr_connect(SR_CONN_DEFAULT, &conn), SR_ERR_OK);
-    assert_int_equal(sr_install_module(conn, module1, NULL, NULL), SR_ERR_OK);
-    assert_int_equal(sr_install_module(conn, module2, NULL, NULL), SR_ERR_OK);
-    assert_int_equal(sr_disconnect(conn), SR_ERR_OK);
+    /* setup environment */
+    rc = np2_glob_test_setup_env(test_name);
+    assert_int_equal(rc, 0);
 
     /* setup netopeer2 server */
-    if (!(rv = np_glob_setup_np2(state, test_name))) {
-        /* state is allocated in np_glob_setup_np2 have to set here */
-        st = *state;
-        /* Open connection to start a sessions for the tests */
-        assert_int_equal(sr_connect(SR_CONN_DEFAULT, &st->conn), SR_ERR_OK);
-        assert_int_equal(sr_session_start(st->conn, SR_DS_RUNNING, &st->sr_sess), SR_ERR_OK);
-        assert_int_equal(sr_session_start(st->conn, SR_DS_OPERATIONAL, &st->sr_sess2), SR_ERR_OK);
-        assert_non_null(st->ctx = sr_get_context(st->conn));
+    rc = np2_glob_test_setup_server(state, test_name, modules, 0);
+    assert_int_equal(rc, 0);
+    st = *state;
 
-        /* Enable replay support */
-        assert_int_equal(SR_ERR_OK, sr_set_module_replay_support(st->conn, "notif1", 1));
-        rv |= setup_nacm(state);
-    }
-    return rv;
+    /* second session */
+    assert_int_equal(sr_session_start(st->conn, SR_DS_OPERATIONAL, &st->sr_sess2), SR_ERR_OK);
+
+    /* enable replay support */
+    assert_int_equal(SR_ERR_OK, sr_set_module_replay_support(st->conn, "notif1", 1));
+
+    /* setup NACM */
+    rc = np2_glob_test_setup_nacm(state);
+    assert_int_equal(rc, 0);
+
+    return 0;
 }
 
 static int
 teardown_common(void **state)
 {
-    struct np_test *st = *state;
-    char *cmd;
-    int ret;
+    struct np2_test *st = *state;
 
-    /* Remove the notifications */
-    if (asprintf(&cmd, "rm -rf %s/%s/data/notif/notif1.notif*", NP_SR_REPOS_DIR, st->test_name) == -1) {
-        return 1;
-    }
-
-    ret = system(cmd);
-    free(cmd);
-
-    if (ret == -1) {
-        return 1;
-    } else if (!WIFEXITED(ret) || WEXITSTATUS(ret)) {
+    /* remove notifications */
+    if (np2_glob_test_teardown_notif(st->test_name)) {
         return 1;
     }
 
     /* reestablish NETCONF connection */
     nc_session_free(st->nc_sess, NULL);
-    st->nc_sess = nc_connect_unix(st->socket_path, NULL);
+    st->nc_sess = nc_connect_unix(st->socket_path, (struct ly_ctx *)nc_session_get_ctx(st->nc_sess2));
     assert_non_null(st->nc_sess);
 
     return 0;
@@ -103,38 +88,30 @@ teardown_common(void **state)
 static int
 local_teardown(void **state)
 {
-    struct np_test *st = *state;
-    sr_conn_ctx_t *conn;
+    struct np2_test *st = *state;
+    const char *modules[] = {"notif1", "notif2", NULL};
 
     if (!st) {
         return 0;
     }
 
-    /* Disable replay support */
+    /* disable replay support */
     assert_int_equal(SR_ERR_OK, sr_set_module_replay_support(st->conn, "notif1", 0));
 
-    /* Close the sessions and connection needed for tests */
-    assert_int_equal(sr_session_stop(st->sr_sess), SR_ERR_OK);
+    /* close the session */
     assert_int_equal(sr_session_stop(st->sr_sess2), SR_ERR_OK);
-    assert_int_equal(sr_disconnect(st->conn), SR_ERR_OK);
-
-    /* connect to server and remove test modules */
-    assert_int_equal(sr_connect(SR_CONN_DEFAULT, &conn), SR_ERR_OK);
-    assert_int_equal(sr_remove_module(conn, "notif1"), SR_ERR_OK);
-    assert_int_equal(sr_remove_module(conn, "notif2"), SR_ERR_OK);
-    assert_int_equal(sr_disconnect(conn), SR_ERR_OK);
 
     /* Remove the notifications */
     teardown_common(state);
 
     /* close netopeer2 server */
-    return np_glob_teardown(state);
+    return np2_glob_test_teardown(state, modules);
 }
 
 static void
 test_filter_pass(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *filter, *data;
 
     filter = "<n1 xmlns=\"n1\"/>";
@@ -150,7 +127,7 @@ test_filter_pass(void **state)
     NOTIF_PARSE(st, data);
 
     /* Send the notification */
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
     RECV_NOTIF(st);
     assert_string_equal(data, st->str);
     FREE_TEST_VARS(st);
@@ -159,7 +136,7 @@ test_filter_pass(void **state)
 static void
 test_filter_no_pass(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *filter, *data;
 
     filter = "<n1 xmlns=\"n1\"><first>Different</first></n1>";
@@ -175,7 +152,7 @@ test_filter_no_pass(void **state)
     NOTIF_PARSE(st, data);
 
     /* Notification should not pass */
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
     ASSERT_NO_NOTIF(st);
     FREE_TEST_VARS(st);
 }
@@ -183,7 +160,7 @@ test_filter_no_pass(void **state)
 static void
 test_modifysub_filter(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *filter, *data, *template;
     char *ntf;
 
@@ -198,7 +175,7 @@ test_modifysub_filter(void **state)
             "  <first>Test</first>\n"
             "</n1>\n";
     NOTIF_PARSE(st, data);
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
     RECV_NOTIF(st);
     assert_string_equal(data, st->str);
     FREE_TEST_VARS(st);
@@ -230,7 +207,7 @@ test_modifysub_filter(void **state)
             "  <first>Test</first>\n"
             "</n1>\n";
     NOTIF_PARSE(st, data);
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
     ASSERT_NO_NOTIF(st);
     FREE_TEST_VARS(st);
 }
@@ -238,7 +215,7 @@ test_modifysub_filter(void **state)
 static void
 test_modifysub_stop_time(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data, *template;
     char *ntf;
     char *stop_time;
@@ -268,7 +245,7 @@ test_modifysub_stop_time(void **state)
     template =
             "<subscription-terminated xmlns=\"urn:ietf:params:xml:ns:yang:ietf-subscribed-notifications\">\n"
             "  <id>%d</id>\n"
-            "  <reason xmlns:sn=\"urn:ietf:params:xml:ns:yang:ietf-subscribed-notifications\">sn:no-such-subscription</reason>\n"
+            "  <reason>no-such-subscription</reason>\n"
             "</subscription-terminated>\n";
     assert_int_not_equal(-1, asprintf(&ntf, template, st->ntf_id));
     assert_string_equal(st->str, ntf);
@@ -281,7 +258,7 @@ test_modifysub_stop_time(void **state)
             "  <first>Test</first>\n"
             "</n1>\n";
     NOTIF_PARSE(st, data);
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
     ASSERT_NO_NOTIF(st);
     FREE_TEST_VARS(st);
 }
@@ -289,15 +266,15 @@ test_modifysub_stop_time(void **state)
 static void
 test_modifysub_fail_no_such_sub(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
 
     /* Try modifying a non-existent subscription */
     SEND_RPC_MODSUB(st, 1, "<n1 xmlns=\"n1\"/>", NULL);
-    ASSERT_RPC_ERROR(st);
+    ASSERT_ERROR_REPLY(st);
     /* Check if correct error-tag */
     assert_string_equal(lyd_get_value(lyd_child(lyd_child(st->envp))->next), "invalid-value");
     /* Check if correct error-app-tag */
-    assert_string_equal(lyd_get_value(lyd_child(lyd_child(st->envp))->next->next->next->next),
+    assert_string_equal(lyd_get_value(lyd_child(lyd_child(st->envp))->next->next->next),
             "ietf-subscribed-notifications:no-such-subscription");
     FREE_TEST_VARS(st);
 }
@@ -305,7 +282,7 @@ test_modifysub_fail_no_such_sub(void **state)
 static void
 test_deletesub(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *template, *data;
     char *ntf;
 
@@ -319,7 +296,7 @@ test_deletesub(void **state)
     template =
             "<subscription-terminated xmlns=\"urn:ietf:params:xml:ns:yang:ietf-subscribed-notifications\">\n"
             "  <id>%d</id>\n"
-            "  <reason xmlns:sn=\"urn:ietf:params:xml:ns:yang:ietf-subscribed-notifications\">sn:no-such-subscription</reason>\n"
+            "  <reason>no-such-subscription</reason>\n"
             "</subscription-terminated>\n";
     assert_int_not_equal(-1, asprintf(&ntf, template, st->ntf_id));
     assert_string_equal(st->str, ntf);
@@ -334,7 +311,7 @@ test_deletesub(void **state)
             "  <first>Test</first>\n"
             "</n1>\n";
     NOTIF_PARSE(st, data);
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
     ASSERT_NO_NOTIF(st);
     FREE_TEST_VARS(st);
 }
@@ -342,15 +319,15 @@ test_deletesub(void **state)
 static void
 test_deletesub_fail(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
 
     /* Try deleting a non-existent subscription */
     SEND_RPC_DELSUB(st, 1);
-    ASSERT_RPC_ERROR(st);
+    ASSERT_ERROR_REPLY(st);
     /* Check if correct error-tag */
     assert_string_equal(lyd_get_value(lyd_child(lyd_child(st->envp))->next), "invalid-value");
     /* Check if correct error-app-tag */
-    assert_string_equal(lyd_get_value(lyd_child(lyd_child(st->envp))->next->next->next->next),
+    assert_string_equal(lyd_get_value(lyd_child(lyd_child(st->envp))->next->next->next),
             "ietf-subscribed-notifications:no-such-subscription");
     FREE_TEST_VARS(st);
 }
@@ -358,7 +335,7 @@ test_deletesub_fail(void **state)
 static void
 test_deletesub_fail_diff_sess(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data;
     struct nc_session *tmp;
 
@@ -373,29 +350,26 @@ test_deletesub_fail_diff_sess(void **state)
             "  <first>Test</first>\n"
             "</n1>\n";
     NOTIF_PARSE(st, data);
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
     FREE_TEST_VARS(st);
     RECV_NOTIF(st);
     assert_string_equal(st->str, data);
     FREE_TEST_VARS(st);
 
     /* Create a new session */
-    tmp = nc_connect_unix(st->socket_path, NULL);
+    tmp = nc_connect_unix(st->socket_path, (struct ly_ctx *)nc_session_get_ctx(st->nc_sess2));
     assert_non_null(tmp);
 
     /* Try to delete it */
     st->rpc = nc_rpc_deletesub(st->ntf_id);
     st->msgtype = nc_send_rpc(tmp, st->rpc, 1000, &st->msgid);
     assert_int_equal(NC_MSG_RPC, st->msgtype);
+
     /* Receive rpc-error reply */
-    st->msgtype = nc_recv_reply(tmp, st->rpc, st->msgid, 2000, &st->envp, &st->op);
-    assert_int_equal(st->msgtype, NC_MSG_REPLY);
-    assert_null(st->op);
-    assert_string_equal(LYD_NAME(lyd_child(st->envp)), "rpc-error");
-    /* Check if correct error-tag */
-    assert_string_equal(lyd_get_value(lyd_child(lyd_child(st->envp))->next), "invalid-value");
+    ASSERT_ERROR_REPLY_PARAM(st, tmp, "invalid-value");
+
     /* Check if correct error-app-tag */
-    assert_string_equal(lyd_get_value(lyd_child(lyd_child(st->envp))->next->next->next->next),
+    assert_string_equal(lyd_get_value(lyd_child(lyd_child(st->envp))->next->next->next),
             "ietf-subscribed-notifications:no-such-subscription");
     FREE_TEST_VARS(st);
 
@@ -406,7 +380,7 @@ test_deletesub_fail_diff_sess(void **state)
 static void
 test_ds_subscriptions(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     char *expected;
     const char *template =
             "<get xmlns=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
@@ -442,7 +416,7 @@ test_ds_subscriptions(void **state)
 static void
 test_ds_subscriptions_sent_event(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     char *expected;
     const char *data, *template =
             "<get xmlns=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
@@ -475,7 +449,7 @@ test_ds_subscriptions_sent_event(void **state)
             "</n1>\n";
     for (uint8_t i = 0; i < 3; i++) {
         NOTIF_PARSE(st, data);
-        assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+        assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
         RECV_NOTIF(st);
         FREE_TEST_VARS(st);
     }
@@ -490,7 +464,7 @@ test_ds_subscriptions_sent_event(void **state)
 static void
 test_ds_subscriptions_excluded_event(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     char *expected;
     const char *data, *filter, *template =
             "<get xmlns=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
@@ -523,7 +497,7 @@ test_ds_subscriptions_excluded_event(void **state)
             "  <first>Different</first>\n"
             "</n1>\n";
     NOTIF_PARSE(st, data);
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
     RECV_NOTIF(st);
     FREE_TEST_VARS(st);
     data =
@@ -531,7 +505,7 @@ test_ds_subscriptions_excluded_event(void **state)
             "  <first>Test</first>\n"
             "</n1>\n";
     NOTIF_PARSE(st, data);
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
     ASSERT_NO_NOTIF(st);
     FREE_TEST_VARS(st);
 
@@ -545,7 +519,7 @@ test_ds_subscriptions_excluded_event(void **state)
 static void
 test_multiple_subscriptions(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     char *expected;
     uint32_t nc_sess_id, tmp_id;
     const char *template =
@@ -600,7 +574,7 @@ test_multiple_subscriptions(void **state)
 static void
 test_multiple_subscriptions_notif(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     char *expected;
     uint32_t nc_sess_id, tmp_ids[3];
     const char *data, *template =
@@ -658,7 +632,7 @@ test_multiple_subscriptions_notif(void **state)
             "  <first>Test</first>\n"
             "</n1>\n";
     NOTIF_PARSE(st, data);
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
     FREE_TEST_VARS(st);
 
     /* Receive three notifications */
@@ -679,7 +653,7 @@ test_multiple_subscriptions_notif(void **state)
 static int
 setup_notif2_data(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data;
 
     data =
@@ -697,8 +671,8 @@ setup_notif2_data(void **state)
 static void
 test_multiple_subscriptions_notif_interlaced(void **state)
 {
-    struct np_test *st = *state;
-    const char *data;
+    struct np2_test *st = *state;
+    const char *data, *data2;
 
     /* Establish first sub */
     SEND_RPC_ESTABSUB(st, NULL, "notif1", NULL, NULL);
@@ -711,7 +685,7 @@ test_multiple_subscriptions_notif_interlaced(void **state)
             "  <first>Test</first>\n"
             "</n1>\n";
     NOTIF_PARSE(st, data);
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
     FREE_TEST_VARS(st);
     RECV_NOTIF(st);
     assert_string_equal(st->str, data);
@@ -720,10 +694,10 @@ test_multiple_subscriptions_notif_interlaced(void **state)
     /* Send another notification to the first session */
     data =
             "<n1 xmlns=\"n1\">\n"
-            "  <first>Test</first>\n"
+            "  <first>Test2</first>\n"
             "</n1>\n";
     NOTIF_PARSE(st, data);
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
     FREE_TEST_VARS(st);
 
     /* Establish second sub for a different stream */
@@ -734,7 +708,7 @@ test_multiple_subscriptions_notif_interlaced(void **state)
     assert_string_equal(st->str, data);
     FREE_TEST_VARS(st);
 
-    /* Check for establishing the sub */
+    /* Check reply of establishing the sub */
     st->rpc = nc_rpc_establishsub(NULL, "notif2", NULL, NULL, NULL, NC_PARAMTYPE_CONST);
     ASSERT_OK_SUB_NTF(st);
     FREE_TEST_VARS(st);
@@ -742,14 +716,14 @@ test_multiple_subscriptions_notif_interlaced(void **state)
     /* Send last notification to the first session */
     data =
             "<n1 xmlns=\"n1\">\n"
-            "  <first>Test</first>\n"
+            "  <first>Test3</first>\n"
             "</n1>\n";
     NOTIF_PARSE(st, data);
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
     FREE_TEST_VARS(st);
 
     /* Send notification to the second session */
-    data =
+    data2 =
             "<devices xmlns=\"n2\">\n"
             "  <device>\n"
             "    <name>Main</name>\n"
@@ -758,38 +732,25 @@ test_multiple_subscriptions_notif_interlaced(void **state)
             "    </power-on>\n"
             "  </device>\n"
             "</devices>\n";
-    NOTIF_PARSE(st, data);
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    NOTIF_PARSE(st, data2);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
     FREE_TEST_VARS(st);
 
     /* Receive the notification from first sub */
     RECV_NOTIF(st);
-    data =
-            "<n1 xmlns=\"n1\">\n"
-            "  <first>Test</first>\n"
-            "</n1>\n";
     assert_string_equal(st->str, data);
     FREE_TEST_VARS(st);
 
     /* Receive the notification from second sub */
     RECV_NOTIF(st);
-    data =
-            "<devices xmlns=\"n2\">\n"
-            "  <device>\n"
-            "    <name>Main</name>\n"
-            "    <power-on>\n"
-            "      <boot-time>12</boot-time>\n"
-            "    </power-on>\n"
-            "  </device>\n"
-            "</devices>\n";
-    assert_string_equal(st->str, data);
+    assert_string_equal(st->str, data2);
     FREE_TEST_VARS(st);
 }
 
 static int
 teardown_nacm(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data;
 
     teardown_common(state);
@@ -813,17 +774,17 @@ teardown_nacm(void **state)
 static void
 test_killsub_fail_nacm(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
 
-    /* Check for NACM_RECOVERY_UID */
-    if (is_nacm_rec_uid()) {
-        puts("Running as NACM_RECOVERY_UID. Tests will not run correctly as this user bypases NACM. Skipping.");
+    /* check for NACM_RECOVERY_USER */
+    if (np2_is_nacm_recovery()) {
+        puts("Running as NACM_RECOVERY_USER. Tests will not run correctly as this user bypases NACM. Skipping.");
         return;
     }
 
     /* Should fail on NACM */
     SEND_RPC_KILLSUB(st, 1);
-    ASSERT_RPC_ERROR(st);
+    ASSERT_ERROR_REPLY(st);
     assert_string_equal(lyd_get_value(lyd_child(lyd_child(st->envp))->next), "access-denied");
     FREE_TEST_VARS(st);
 }
@@ -831,7 +792,7 @@ test_killsub_fail_nacm(void **state)
 static int
 setup_test_killsub(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data =
             "<nacm xmlns=\"urn:ietf:params:xml:ns:yang:ietf-netconf-acm\">\n"
             "  <rule-list>\n"
@@ -855,15 +816,15 @@ setup_test_killsub(void **state)
 static void
 test_killsub_fail_no_such_sub(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
 
     SEND_RPC_KILLSUB(st, 1);
     /* Should fail on no such sub */
-    ASSERT_RPC_ERROR(st);
+    ASSERT_ERROR_REPLY(st);
     /* Check if correct error-tag */
     assert_string_equal(lyd_get_value(lyd_child(lyd_child(st->envp))->next), "invalid-value");
     /* Check if correct error-app-tag */
-    assert_string_equal(lyd_get_value(lyd_child(lyd_child(st->envp))->next->next->next->next),
+    assert_string_equal(lyd_get_value(lyd_child(lyd_child(st->envp))->next->next->next),
             "ietf-subscribed-notifications:no-such-subscription");
     FREE_TEST_VARS(st);
 }
@@ -871,7 +832,7 @@ test_killsub_fail_no_such_sub(void **state)
 static void
 test_killsub_same_sess(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data, *template;
     char *ntf;
 
@@ -886,7 +847,7 @@ test_killsub_same_sess(void **state)
             "  <first>Test</first>\n"
             "</n1>\n";
     NOTIF_PARSE(st, data);
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
     FREE_TEST_VARS(st);
     RECV_NOTIF(st);
     assert_string_equal(st->str, data);
@@ -899,7 +860,7 @@ test_killsub_same_sess(void **state)
     template =
             "<subscription-terminated xmlns=\"urn:ietf:params:xml:ns:yang:ietf-subscribed-notifications\">\n"
             "  <id>%d</id>\n"
-            "  <reason xmlns:sn=\"urn:ietf:params:xml:ns:yang:ietf-subscribed-notifications\">sn:no-such-subscription</reason>\n"
+            "  <reason>no-such-subscription</reason>\n"
             "</subscription-terminated>\n";
     assert_int_not_equal(-1, asprintf(&ntf, template, st->ntf_id));
     assert_string_equal(st->str, ntf);
@@ -915,7 +876,7 @@ test_killsub_same_sess(void **state)
             "  <first>Test</first>\n"
             "</n1>\n";
     NOTIF_PARSE(st, data);
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
     FREE_TEST_VARS(st);
     ASSERT_NO_NOTIF(st);
     FREE_TEST_VARS(st);
@@ -924,7 +885,7 @@ test_killsub_same_sess(void **state)
 static void
 test_killsub_diff_sess(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data, *template;
     struct nc_session *tmp;
     char *ntf;
@@ -940,14 +901,14 @@ test_killsub_diff_sess(void **state)
             "  <first>Test</first>\n"
             "</n1>\n";
     NOTIF_PARSE(st, data);
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
     FREE_TEST_VARS(st);
     RECV_NOTIF(st);
     assert_string_equal(st->str, data);
     FREE_TEST_VARS(st);
 
     /* Create a new session */
-    tmp = nc_connect_unix(st->socket_path, NULL);
+    tmp = nc_connect_unix(st->socket_path, (struct ly_ctx *)nc_session_get_ctx(st->nc_sess2));
     assert_non_null(tmp);
 
     /* Kill it */
@@ -959,18 +920,16 @@ test_killsub_diff_sess(void **state)
     template =
             "<subscription-terminated xmlns=\"urn:ietf:params:xml:ns:yang:ietf-subscribed-notifications\">\n"
             "  <id>%d</id>\n"
-            "  <reason xmlns:sn=\"urn:ietf:params:xml:ns:yang:ietf-subscribed-notifications\">sn:no-such-subscription</reason>\n"
+            "  <reason>no-such-subscription</reason>\n"
             "</subscription-terminated>\n";
     assert_int_not_equal(-1, asprintf(&ntf, template, st->ntf_id));
     assert_string_equal(st->str, ntf);
     free(ntf);
     FREE_TEST_VARS(st);
     st->rpc = nc_rpc_killsub(st->ntf_id);
+
     /* Receive OK reply */
-    st->msgtype = nc_recv_reply(tmp, st->rpc, st->msgid, 2000, &st->envp, &st->op);
-    assert_int_equal(st->msgtype, NC_MSG_REPLY);
-    assert_null(st->op);
-    assert_string_equal(LYD_NAME(lyd_child(st->envp)), "ok");
+    ASSERT_OK_REPLY_SESS(st, tmp);
     FREE_TEST_VARS(st);
 
     /* Send notification, should NOT arrive */
@@ -979,7 +938,7 @@ test_killsub_diff_sess(void **state)
             "  <first>Test</first>\n"
             "</n1>\n";
     NOTIF_PARSE(st, data);
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
     FREE_TEST_VARS(st);
     ASSERT_NO_NOTIF(st);
     FREE_TEST_VARS(st);
