@@ -18,8 +18,10 @@
 
 #include <setjmp.h>
 #include <stdarg.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/wait.h>
 #include <time.h>
 #include <unistd.h>
 
@@ -28,97 +30,70 @@
 #include <nc_client.h>
 #include <sysrepo.h>
 
-#include "np_test.h"
-#include "np_test_config.h"
+#include "np2_test.h"
+#include "np2_test_config.h"
 
 static void
 reestablish_sub(void **state, const char *stream, const char *start_time, const char *stop_time)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
 
-    /* Reestablish NETCONF connection */
+    /* free the current session (with its subscription) */
     nc_session_free(st->nc_sess, NULL);
-    st->nc_sess = nc_connect_unix(st->socket_path, NULL);
+
+    /* because of multithreading, try to prevent netconf-session-end being generated AFTER the new subscription is made */
+    usleep(20000);
+
+    /* create a new session */
+    st->nc_sess = nc_connect_unix(st->socket_path, (struct ly_ctx *)nc_session_get_ctx(st->nc_sess2));
     assert_non_null(st->nc_sess);
 
-    /* Get a subscription to receive notifications */
+    /* get a subscription to receive notifications */
     st->rpc = nc_rpc_subscribe(stream, NULL, start_time, stop_time, NC_PARAMTYPE_CONST);
     st->msgtype = nc_send_rpc(st->nc_sess, st->rpc, 1000, &st->msgid);
     assert_int_equal(NC_MSG_RPC, st->msgtype);
 
-    /* Check reply */
-    st->msgtype = NC_MSG_NOTIF;
-    while (st->msgtype == NC_MSG_NOTIF) {
-        st->msgtype = nc_recv_reply(st->nc_sess, st->rpc, st->msgid, 1000, &st->envp, &st->op);
-    }
-    assert_int_equal(NC_MSG_REPLY, st->msgtype);
-    assert_null(st->op);
-    if (strcmp(LYD_NAME(lyd_child(st->envp)), "ok")) {
-        lyd_print_file(stdout, st->envp, LYD_XML, 0);
-        fprintf(stdout, "start time: %s\n", start_time);
-        fprintf(stdout, "stop time:  %s\n", stop_time);
-        fail();
-    }
-
+    /* check reply */
+    ASSERT_OK_REPLY(st);
     FREE_TEST_VARS(st);
 }
 
 static int
 local_setup(void **state)
 {
-    struct np_test *st;
-    sr_conn_ctx_t *conn;
+    struct np2_test *st;
+    const char *modules[] = {NP_TEST_MODULE_DIR "/notif1.yang", NP_TEST_MODULE_DIR "/notif2.yang", NULL};
     char test_name[256];
-    const char *module1 = NP_TEST_MODULE_DIR "/notif1.yang";
-    const char *module2 = NP_TEST_MODULE_DIR "/notif2.yang";
-    int rv;
+    int rc;
 
     /* get test name */
-    np_glob_setup_test_name(test_name);
+    np2_glob_test_setup_test_name(test_name);
 
-    /* Setup environment necessary for installing module */
-    rv = np_glob_setup_env(test_name);
-    assert_int_equal(rv, 0);
+    /* setup environment */
+    rc = np2_glob_test_setup_env(test_name);
+    assert_int_equal(rc, 0);
 
-    /* connect to server and install test modules */
-    assert_int_equal(sr_connect(SR_CONN_DEFAULT, &conn), SR_ERR_OK);
-    assert_int_equal(sr_install_module(conn, module1, NULL, NULL), SR_ERR_OK);
-    assert_int_equal(sr_install_module(conn, module2, NULL, NULL), SR_ERR_OK);
-    assert_int_equal(sr_disconnect(conn), SR_ERR_OK);
+    /* setup netopeer2 server */
+    rc = np2_glob_test_setup_server(state, test_name, modules, 0);
+    assert_int_equal(rc, 0);
+    st = *state;
 
-    /* Setup netopeer2 server */
-    if (!(rv = np_glob_setup_np2(state, test_name))) {
-        /* State is allocated in np_glob_setup_np2 have to set here */
-        st = *state;
-        /* Open connection to start a session for the tests */
-        assert_int_equal(sr_connect(SR_CONN_DEFAULT, &st->conn), SR_ERR_OK);
-        assert_int_equal(sr_session_start(st->conn, SR_DS_RUNNING, &st->sr_sess), SR_ERR_OK);
-        assert_int_equal(sr_session_start(st->conn, SR_DS_RUNNING, &st->sr_sess2), SR_ERR_OK);
-        assert_non_null(st->ctx = sr_get_context(st->conn));
+    /* second running session */
+    assert_int_equal(sr_session_start(st->conn, SR_DS_RUNNING, &st->sr_sess2), SR_ERR_OK);
 
-        /* Enable replay support */
-        assert_int_equal(SR_ERR_OK, sr_set_module_replay_support(st->conn, "notif1", 1));
-    }
-    return rv;
+    /* rnable replay support */
+    assert_int_equal(SR_ERR_OK, sr_set_module_replay_support(st->conn, "notif1", 1));
+
+    return 0;
 }
 
 static int
 clear_notif(void **state)
 {
-    struct np_test *st = *state;
-    char *cmd;
-    int ret;
+    struct np2_test *st = *state;
 
-    if (asprintf(&cmd, "rm -rf %s/%s/data/notif/notif1.notif*", NP_SR_REPOS_DIR, st->test_name) == -1) {
-        return 1;
-    }
-
-    ret = system(cmd);
-    free(cmd);
-
-    if (ret == -1) {
-        return 1;
-    } else if (!WIFEXITED(ret) || WEXITSTATUS(ret)) {
+    /* remove notifications */
+    if (np2_glob_test_teardown_notif(st->test_name)) {
         return 1;
     }
 
@@ -128,39 +103,30 @@ clear_notif(void **state)
 static int
 local_teardown(void **state)
 {
-    struct np_test *st = *state;
-    sr_conn_ctx_t *conn;
+    struct np2_test *st = *state;
+    const char *modules[] = {"notif1", "notif2", NULL};
 
     if (!st) {
         return 0;
     }
 
-    /* Disable replay support */
+    /* disable replay support */
     assert_int_equal(SR_ERR_OK, sr_set_module_replay_support(st->conn, "notif1", 0));
 
-    /* Close the sessions and connection needed for tests */
-    assert_int_equal(sr_session_stop(st->sr_sess), SR_ERR_OK);
+    /* close the session */
     assert_int_equal(sr_session_stop(st->sr_sess2), SR_ERR_OK);
-    assert_int_equal(sr_disconnect(st->conn), SR_ERR_OK);
 
-    /* Connect to server and remove test modules */
-    assert_int_equal(sr_connect(SR_CONN_DEFAULT, &conn), SR_ERR_OK);
-    assert_int_equal(sr_remove_module(conn, "notif1"), SR_ERR_OK);
-    assert_int_equal(sr_remove_module(conn, "notif2"), SR_ERR_OK);
-    assert_int_equal(sr_disconnect(conn), SR_ERR_OK);
-
-    /* Remove the notfications */
+    /* remove the notfications */
     clear_notif(state);
 
     /* close netopeer2 server */
-    return np_glob_teardown(state);
+    return np2_glob_test_teardown(state, modules);
 }
 
 static void
 test_stop_time_invalid(void **state)
 {
-    struct np_test *st = *state;
-    const char *expected;
+    struct np2_test *st = *state;
     struct timespec ts;
     char *start_time, *stop_time;
 
@@ -174,7 +140,7 @@ test_stop_time_invalid(void **state)
 
     /* Reestablish NETCONF connection */
     nc_session_free(st->nc_sess, NULL);
-    st->nc_sess = nc_connect_unix(st->socket_path, NULL);
+    st->nc_sess = nc_connect_unix(st->socket_path, (struct ly_ctx *)nc_session_get_ctx(st->nc_sess2));
     assert_non_null(st->nc_sess);
 
     /* Get a subscription to receive notifications */
@@ -183,23 +149,17 @@ test_stop_time_invalid(void **state)
     assert_int_equal(NC_MSG_RPC, st->msgtype);
 
     /* Check reply */
-    st->msgtype = NC_MSG_NOTIF;
-    while (st->msgtype == NC_MSG_NOTIF) {
-        st->msgtype = nc_recv_reply(st->nc_sess, st->rpc, st->msgid, 1000, &st->envp, &st->op);
-    }
-    assert_int_equal(NC_MSG_REPLY, st->msgtype);
-    assert_null(st->op);
-    assert_string_equal(LYD_NAME(lyd_child(st->envp)), "rpc-error");
-
-    /* Check if correct error-tag and error-info */
-    assert_string_equal(lyd_get_value(lyd_child(lyd_child(st->envp))->next), "bad-element");
-    expected =
-            "    <error-info>\n"
-            "      <bad-element>stopTime</bad-element>\n"
-            "    </error-info>\n";
-    lyd_print_mem(&st->str, st->envp, LYD_XML, 0);
-    assert_non_null(strstr(st->str, expected));
-
+    ASSERT_ERROR_REPLY(st);
+    assert_string_equal(st->str,
+            "<rpc-error xmlns=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
+            "  <error-type>protocol</error-type>\n"
+            "  <error-tag>bad-element</error-tag>\n"
+            "  <error-severity>error</error-severity>\n"
+            "  <error-message xml:lang=\"en\">Specified \"stopTime\" is earlier than \"startTime\".</error-message>\n"
+            "  <error-info>\n"
+            "    <bad-element>stopTime</bad-element>\n"
+            "  </error-info>\n"
+            "</rpc-error>\n");
     FREE_TEST_VARS(st);
     free(start_time);
     free(stop_time);
@@ -208,19 +168,18 @@ test_stop_time_invalid(void **state)
 static void
 test_start_time_invalid(void **state)
 {
-    struct np_test *st = *state;
-    const char *expected;
+    struct np2_test *st = *state;
     struct timespec ts;
     char *start_time;
 
     /* startTime is in the future */
     assert_int_not_equal(-1, clock_gettime(CLOCK_REALTIME, &ts));
-    ts.tv_sec += 10;
+    ts.tv_sec += 30;
     assert_int_equal(LY_SUCCESS, ly_time_ts2str(&ts, &start_time));
 
     /* Reestablish NETCONF connection */
     nc_session_free(st->nc_sess, NULL);
-    st->nc_sess = nc_connect_unix(st->socket_path, NULL);
+    st->nc_sess = nc_connect_unix(st->socket_path, (struct ly_ctx *)nc_session_get_ctx(st->nc_sess2));
     assert_non_null(st->nc_sess);
 
     /* Get a subscription to receive notifications */
@@ -229,23 +188,17 @@ test_start_time_invalid(void **state)
     assert_int_equal(NC_MSG_RPC, st->msgtype);
 
     /* Check reply */
-    st->msgtype = NC_MSG_NOTIF;
-    while (st->msgtype == NC_MSG_NOTIF) {
-        st->msgtype = nc_recv_reply(st->nc_sess, st->rpc, st->msgid, 1000, &st->envp, &st->op);
-    }
-    assert_int_equal(NC_MSG_REPLY, st->msgtype);
-    assert_null(st->op);
-    assert_string_equal(LYD_NAME(lyd_child(st->envp)), "rpc-error");
-
-    /* Check if correct error-tag and error-info */
-    assert_string_equal(lyd_get_value(lyd_child(lyd_child(st->envp))->next), "bad-element");
-    expected =
-            "    <error-info>\n"
-            "      <bad-element>startTime</bad-element>\n"
-            "    </error-info>\n";
-    lyd_print_mem(&st->str, st->envp, LYD_XML, 0);
-    assert_non_null(strstr(st->str, expected));
-
+    ASSERT_ERROR_REPLY(st);
+    assert_string_equal(st->str,
+            "<rpc-error xmlns=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
+            "  <error-type>protocol</error-type>\n"
+            "  <error-tag>bad-element</error-tag>\n"
+            "  <error-severity>error</error-severity>\n"
+            "  <error-message xml:lang=\"en\">Specified \"startTime\" is in future.</error-message>\n"
+            "  <error-info>\n"
+            "    <bad-element>startTime</bad-element>\n"
+            "  </error-info>\n"
+            "</rpc-error>\n");
     FREE_TEST_VARS(st);
     free(start_time);
 }
@@ -253,8 +206,7 @@ test_start_time_invalid(void **state)
 static void
 test_stop_time_no_start_time(void **state)
 {
-    struct np_test *st = *state;
-    const char *expected;
+    struct np2_test *st = *state;
     struct timespec ts;
     char *stop_time;
 
@@ -264,7 +216,7 @@ test_stop_time_no_start_time(void **state)
 
     /* Reestablish NETCONF connection */
     nc_session_free(st->nc_sess, NULL);
-    st->nc_sess = nc_connect_unix(st->socket_path, NULL);
+    st->nc_sess = nc_connect_unix(st->socket_path, (struct ly_ctx *)nc_session_get_ctx(st->nc_sess2));
     assert_non_null(st->nc_sess);
 
     /* Get a subscription to receive notifications */
@@ -273,23 +225,17 @@ test_stop_time_no_start_time(void **state)
     assert_int_equal(NC_MSG_RPC, st->msgtype);
 
     /* Check reply */
-    st->msgtype = NC_MSG_NOTIF;
-    while (st->msgtype == NC_MSG_NOTIF) {
-        st->msgtype = nc_recv_reply(st->nc_sess, st->rpc, st->msgid, 1000, &st->envp, &st->op);
-    }
-    assert_int_equal(NC_MSG_REPLY, st->msgtype);
-    assert_null(st->op);
-    assert_string_equal(LYD_NAME(lyd_child(st->envp)), "rpc-error");
-
-    /* Check if correct error-tag and error-info */
-    assert_string_equal(lyd_get_value(lyd_child(lyd_child(st->envp))->next), "missing-element");
-    expected =
-            "    <error-info>\n"
-            "      <bad-element>startTime</bad-element>\n"
-            "    </error-info>\n";
-    lyd_print_mem(&st->str, st->envp, LYD_XML, 0);
-    assert_non_null(strstr(st->str, expected));
-
+    ASSERT_ERROR_REPLY(st);
+    assert_string_equal(st->str,
+            "<rpc-error xmlns=\"urn:ietf:params:xml:ns:netconf:base:1.0\">\n"
+            "  <error-type>protocol</error-type>\n"
+            "  <error-tag>missing-element</error-tag>\n"
+            "  <error-severity>error</error-severity>\n"
+            "  <error-message xml:lang=\"en\">An expected element is missing.</error-message>\n"
+            "  <error-info>\n"
+            "    <bad-element>startTime</bad-element>\n"
+            "  </error-info>\n"
+            "</rpc-error>\n");
     FREE_TEST_VARS(st);
     free(stop_time);
 }
@@ -297,7 +243,7 @@ test_stop_time_no_start_time(void **state)
 static void
 test_basic_replay(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data;
     struct timespec ts;
     char *start_time;
@@ -312,7 +258,7 @@ test_basic_replay(void **state)
             "  <first>Test</first>\n"
             "</n1>\n";
     NOTIF_PARSE(st, data);
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
 
     /* Subscribe to notfications */
     reestablish_sub(state, NULL, start_time, NULL);
@@ -337,7 +283,7 @@ test_basic_replay(void **state)
 static void
 test_replay_real_time(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data, *expected;
     struct timespec ts;
     char *timestr;
@@ -353,7 +299,7 @@ test_replay_real_time(void **state)
             "</n1>\n";
     expected = data;
     NOTIF_PARSE(st, data);
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
 
     /* Subscribe to notfications */
     reestablish_sub(state, NULL, timestr, NULL);
@@ -365,7 +311,7 @@ test_replay_real_time(void **state)
             "  <first>Second</first>\n"
             "</n1>\n";
     NOTIF_PARSE(st, data);
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
 
     /* Receive the notification and test the contents */
     RECV_NOTIF(st);
@@ -395,7 +341,7 @@ test_replay_real_time(void **state)
 static void
 test_stop_time(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data, *expected;
     struct timespec start, stop;
     char *start_time, *stop_time;
@@ -413,7 +359,7 @@ test_stop_time(void **state)
     NOTIF_PARSE(st, data);
 
     /* Send the notification */
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
     FREE_TEST_VARS(st);
 
     /* To subscribe to replay of notifications until time was called, should not include any called after */
@@ -448,7 +394,7 @@ test_stop_time(void **state)
             "  <first>Another</first>\n"
             "</n1>\n";
     NOTIF_PARSE(st, data);
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
     FREE_TEST_VARS(st);
 
     /* No other notification should arrive */
@@ -459,12 +405,12 @@ test_stop_time(void **state)
 static void
 test_stop_time_sub_end(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data;
     struct timespec ts;
     char *start_time, *stop_time;
 
-    /* Needed for stopTime */
+    /* needed for stopTime */
     assert_int_not_equal(-1, clock_gettime(CLOCK_REALTIME, &ts));
     assert_int_equal(LY_SUCCESS, ly_time_ts2str(&ts, &start_time));
 
@@ -472,49 +418,46 @@ test_stop_time_sub_end(void **state)
     assert_int_not_equal(-1, clock_gettime(CLOCK_REALTIME, &ts));
     assert_int_equal(LY_SUCCESS, ly_time_ts2str(&ts, &stop_time));
 
-    /* Subscribe to notfications */
+    /* subscribe to notfications */
     reestablish_sub(state, NULL, start_time, stop_time);
     free(start_time);
     free(stop_time);
 
-    /* Receive the notification and test the contents */
+    /* receive the notification and test the contents */
     RECV_NOTIF(st);
     data = "<replayComplete xmlns=\"urn:ietf:params:xml:ns:netmod:notification\"/>\n";
     assert_string_equal(data, st->str);
     FREE_TEST_VARS(st);
 
-    /* Receive the notification and test the contents */
+    /* receive the notification and test the contents */
     RECV_NOTIF(st);
     data = "<notificationComplete xmlns=\"urn:ietf:params:xml:ns:netmod:notification\"/>\n";
     assert_string_equal(data, st->str);
     FREE_TEST_VARS(st);
 
-    /* Subsription should have ended now due to stop time, try to create a new one */
+    /* wait a bit to increase chances that the subscription thread was scheduled and the subscription finished due to stop time */
+    usleep(10000);
+
+    /* create new subscription */
     st->rpc = nc_rpc_subscribe(NULL, NULL, NULL, NULL, NC_PARAMTYPE_CONST);
     st->msgtype = nc_send_rpc(st->nc_sess, st->rpc, 1000, &st->msgid);
     assert_int_equal(NC_MSG_RPC, st->msgtype);
 
-    /* Check reply */
-    st->msgtype = NC_MSG_NOTIF;
-    while (st->msgtype == NC_MSG_NOTIF) {
-        st->msgtype = nc_recv_reply(st->nc_sess, st->rpc, st->msgid, 1000, &st->envp, &st->op);
-    }
-    assert_int_equal(NC_MSG_REPLY, st->msgtype);
-    assert_null(st->op);
-    assert_string_equal(LYD_NAME(lyd_child(st->envp)), "ok");
+    /* check reply */
+    ASSERT_OK_REPLY(st);
     FREE_TEST_VARS(st);
 
-    /* Try sending a notfication real time on new session */
-    /* Send the notification */
+    /* try sending a notfication real time on new session */
+    /* send the notification */
     data =
             "<n1 xmlns=\"n1\">\n"
             "  <first>Second</first>\n"
             "</n1>\n";
     NOTIF_PARSE(st, data);
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
     FREE_TEST_VARS(st);
 
-    /* Receive the notification and test the contents */
+    /* receive the notification and test the contents */
     RECV_NOTIF(st);
     assert_string_equal(data, st->str);
     FREE_TEST_VARS(st);
@@ -523,7 +466,7 @@ test_stop_time_sub_end(void **state)
 static void
 test_history_only(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data;
     struct timespec ts;
     char *start_time, *stop_time;
@@ -558,7 +501,7 @@ test_history_only(void **state)
 static void
 test_stream_no_pass(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data;
 
     /* Subscribe to notfications from a different stream */
@@ -570,7 +513,7 @@ test_stream_no_pass(void **state)
             "  <first>Test</first>\n"
             "</n1>\n";
     NOTIF_PARSE(st, data);
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
 
     /* It should no be recieved since it is in a a different stream than subscribed to */
     ASSERT_NO_NOTIF(st);
@@ -580,7 +523,7 @@ test_stream_no_pass(void **state)
 static void
 test_stream_pass(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data;
 
     /* Subscribe to notfications from the same stream */
@@ -592,7 +535,7 @@ test_stream_pass(void **state)
             "  <first>Test</first>\n"
             "</n1>\n";
     NOTIF_PARSE(st, data);
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
 
     /* Receive the notification and test the contents */
     RECV_NOTIF(st);
@@ -603,7 +546,7 @@ test_stream_pass(void **state)
 static void
 test_stream_no_pass_start_time(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data;
     struct timespec ts;
     char *start_time;
@@ -618,7 +561,7 @@ test_stream_no_pass_start_time(void **state)
             "  <first>Test</first>\n"
             "</n1>\n";
     NOTIF_PARSE(st, data);
-    assert_int_equal(sr_event_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
+    assert_int_equal(sr_notif_send_tree(st->sr_sess, st->node, 1000, 1), SR_ERR_OK);
     FREE_TEST_VARS(st);
 
     /* Subscribe to notfications from a different stream */
