@@ -18,8 +18,10 @@
 
 #include <setjmp.h>
 #include <stdarg.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/wait.h>
 #include <time.h>
 #include <unistd.h>
 
@@ -28,54 +30,40 @@
 #include <nc_client.h>
 #include <sysrepo.h>
 
-#include "np_test.h"
-#include "np_test_config.h"
+#include "np2_test.h"
+#include "np2_test_config.h"
 
 static int
 local_setup(void **state)
 {
-    struct np_test *st;
-    sr_conn_ctx_t *conn;
+    struct np2_test *st;
+    const char *modules[] = {NP_TEST_MODULE_DIR "/edit1.yang", NP_TEST_MODULE_DIR "/edit2.yang", NULL};
     char test_name[256];
-    const char *module1 = NP_TEST_MODULE_DIR "/edit1.yang";
-    const char *module2 = NP_TEST_MODULE_DIR "/edit2.yang";
-    int rv;
+    int rc;
 
     /* get test name */
-    np_glob_setup_test_name(test_name);
+    np2_glob_test_setup_test_name(test_name);
 
-    /* setup environment necessary for installing module */
-    rv = np_glob_setup_env(test_name);
-    assert_int_equal(rv, 0);
-
-    /* connect to server and install test modules */
-    assert_int_equal(sr_connect(SR_CONN_DEFAULT, &conn), SR_ERR_OK);
-    assert_int_equal(sr_install_module(conn, module1, NULL, NULL), SR_ERR_OK);
-    assert_int_equal(sr_install_module(conn, module2, NULL, NULL), SR_ERR_OK);
-    assert_int_equal(sr_disconnect(conn), SR_ERR_OK);
+    /* setup environment */
+    rc = np2_glob_test_setup_env(test_name);
+    assert_int_equal(rc, 0);
 
     /* setup netopeer2 server */
-    if (!(rv = np_glob_setup_np2(state, test_name))) {
-        /* state is allocated in np_glob_setup_np2 have to set here */
-        st = *state;
-        /* Open connection to start a session for the tests */
-        assert_int_equal(sr_connect(SR_CONN_DEFAULT, &st->conn), SR_ERR_OK);
-        assert_int_equal(sr_session_start(st->conn, SR_DS_RUNNING, &st->sr_sess), SR_ERR_OK);
-        assert_non_null(st->ctx = sr_get_context(st->conn));
+    rc = np2_glob_test_setup_server(state, test_name, modules, 0);
+    assert_int_equal(rc, 0);
+    st = *state;
 
-        /* Enable replay support */
-        assert_int_equal(SR_ERR_OK, sr_set_module_replay_support(st->conn, "edit1", 1));
-    }
-    return rv;
+    /* enable replay support */
+    assert_int_equal(SR_ERR_OK, sr_set_module_replay_support(st->conn, "edit1", 1));
+
+    return 0;
 }
 
 static int
 teardown_common(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data;
-    char *cmd;
-    int ret;
 
     /* stop NETCONF session */
     nc_session_free(st->nc_sess, NULL);
@@ -89,22 +77,13 @@ teardown_common(void **state)
     SR_EDIT(st, data);
     FREE_TEST_VARS(st);
 
-    /* Remove the notifications */
-    if (asprintf(&cmd, "rm -rf %s/%s/data/notif/notif1.notif*", NP_SR_REPOS_DIR, st->test_name) == -1) {
-        return 1;
-    }
-
-    ret = system(cmd);
-    free(cmd);
-
-    if (ret == -1) {
-        return 1;
-    } else if (!WIFEXITED(ret) || WEXITSTATUS(ret)) {
+    /* remove notifications */
+    if (np2_glob_test_teardown_notif(st->test_name)) {
         return 1;
     }
 
     /* create new default NETCONF session */
-    st->nc_sess = nc_connect_unix(st->socket_path, NULL);
+    st->nc_sess = nc_connect_unix(st->socket_path, (struct ly_ctx *)nc_session_get_ctx(st->nc_sess2));
     assert_non_null(st->nc_sess);
 
     return 0;
@@ -113,46 +92,35 @@ teardown_common(void **state)
 static int
 local_teardown(void **state)
 {
-    struct np_test *st = *state;
-    sr_conn_ctx_t *conn;
+    struct np2_test *st = *state;
+    const char *modules[] = {"edit1", "edit2", NULL};
 
     if (!st) {
         return 0;
     }
 
-    /* Disable replay support */
+    /* disable replay support */
     assert_int_equal(SR_ERR_OK, sr_set_module_replay_support(st->conn, "edit1", 0));
 
-    /* Close the sessions and connection needed for tests */
-    assert_int_equal(sr_session_stop(st->sr_sess), SR_ERR_OK);
-    assert_int_equal(sr_session_stop(st->sr_sess2), SR_ERR_OK);
-    assert_int_equal(sr_disconnect(st->conn), SR_ERR_OK);
-
-    /* connect to server and remove test modules */
-    assert_int_equal(sr_connect(SR_CONN_DEFAULT, &conn), SR_ERR_OK);
-    assert_int_equal(sr_remove_module(conn, "edit1"), SR_ERR_OK);
-    assert_int_equal(sr_remove_module(conn, "edit2"), SR_ERR_OK);
-    assert_int_equal(sr_disconnect(conn), SR_ERR_OK);
-
     /* close netopeer2 server */
-    return np_glob_teardown(state);
+    return np2_glob_test_teardown(state, modules);
 }
 
 static void
 test_periodic_basic(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *template, *data;
     char *ntf;
 
-    /* Establish periodic push */
-    st->rpc = nc_rpc_establishpush_periodic("ietf-datastores:running", NULL, NULL, NULL, 10, NULL, NC_PARAMTYPE_CONST);
+    /* establish periodic push */
+    st->rpc = nc_rpc_establishpush_periodic("ietf-datastores:running", "/edit1:*", NULL, NULL, 10, NULL, NC_PARAMTYPE_CONST);
     st->msgtype = nc_send_rpc(st->nc_sess, st->rpc, 1000, &st->msgid);
     assert_int_equal(st->msgtype, NC_MSG_RPC);
     ASSERT_OK_SUB_NTF(st);
     FREE_TEST_VARS(st);
 
-    /* Receive a notification */
+    /* receive a notification */
     RECV_NOTIF(st);
     template =
             "<push-update xmlns=\"urn:ietf:params:xml:ns:yang:ietf-yang-push\">\n"
@@ -161,16 +129,23 @@ test_periodic_basic(void **state)
             "</push-update>\n";
     assert_int_not_equal(-1, asprintf(&ntf, template, st->ntf_id));
     assert_string_equal(st->str, ntf);
-    free(ntf);
     FREE_TEST_VARS(st);
 
-    /* Put some data into the datastore */
+    /* put some data into the datastore */
     data = "<first xmlns=\"ed1\">TestFirst</first>";
     SR_EDIT(st, data);
     FREE_TEST_VARS(st);
 
-    /* Receive a notification with the new data */
+    /* receive a notification */
     RECV_NOTIF(st);
+    if (!strcmp(st->str, ntf)) {
+        /* rare result of a data race when a notification arrived still with the previous data */
+        FREE_TEST_VARS(st);
+
+        /* receive next notification, now it must have the new data (unless the edit took longer than is
+         * the subscription period, hope not) */
+        RECV_NOTIF(st);
+    }
     template =
             "<push-update xmlns=\"urn:ietf:params:xml:ns:yang:ietf-yang-push\">\n"
             "  <id>%d</id>\n"
@@ -178,11 +153,11 @@ test_periodic_basic(void **state)
             "    <first xmlns=\"ed1\">TestFirst</first>\n"
             "  </datastore-contents>\n"
             "</push-update>\n";
+    free(ntf);
     assert_int_not_equal(-1, asprintf(&ntf, template, st->ntf_id));
-    assert_string_equal(st->str, ntf);
     FREE_TEST_VARS(st);
 
-    /* Test yet again if arives with the same data */
+    /* test yet again if arives with the same data */
     RECV_NOTIF(st);
     assert_string_equal(st->str, ntf);
     FREE_TEST_VARS(st);
@@ -192,7 +167,7 @@ test_periodic_basic(void **state)
 static void
 test_on_change_basic(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *template, *data;
     char *ntf;
 
@@ -242,7 +217,7 @@ test_on_change_basic(void **state)
 static void
 test_on_change_multiple(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *template, *data;
     char *ntf;
 
@@ -347,12 +322,12 @@ test_on_change_multiple(void **state)
 static void
 test_periodic_anchor_time(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *template;
     char *ntf;
 
     /* Establish periodic push with anchor-time */
-    st->rpc = nc_rpc_establishpush_periodic("ietf-datastores:running", NULL, NULL, NULL, 10,
+    st->rpc = nc_rpc_establishpush_periodic("ietf-datastores:running", "/edit1:*", NULL, NULL, 10,
             "1970-01-01T01:00:00+01:00", NC_PARAMTYPE_CONST);
     st->msgtype = nc_send_rpc(st->nc_sess, st->rpc, 1000, &st->msgid);
     assert_int_equal(st->msgtype, NC_MSG_RPC);
@@ -377,12 +352,12 @@ test_periodic_anchor_time(void **state)
 static void
 test_on_change_dampening_time(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *template, *data;
     char *ntf;
 
-    /* Establish on-change push with 0.1s dampening time */
-    st->rpc = nc_rpc_establishpush_onchange("ietf-datastores:running", NULL, NULL, NULL, 10, 0,
+    /* Establish on-change push with 0.2s dampening time */
+    st->rpc = nc_rpc_establishpush_onchange("ietf-datastores:running", NULL, NULL, NULL, 20, 0,
             NULL, NC_PARAMTYPE_CONST);
     st->msgtype = nc_send_rpc(st->nc_sess, st->rpc, 1000, &st->msgid);
     assert_int_equal(st->msgtype, NC_MSG_RPC);
@@ -473,7 +448,7 @@ test_on_change_dampening_time(void **state)
 static void
 test_on_change_dampening_time_same_node(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *template, *data;
     char *ntf;
 
@@ -555,7 +530,7 @@ test_on_change_dampening_time_same_node(void **state)
 static void
 test_on_change_dampening_time_create_delete(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *template, *data;
     char *ntf;
 
@@ -640,7 +615,7 @@ test_on_change_dampening_time_create_delete(void **state)
 static void
 test_on_change_excluded(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *template, *data;
     const char *excluded[] = {"create", NULL};
     char *ntf;
@@ -697,12 +672,12 @@ test_on_change_excluded(void **state)
 static void
 test_sync_on_start(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *template;
     char *ntf;
 
     /* Establish on-change push with sync on start */
-    st->rpc = nc_rpc_establishpush_onchange("ietf-datastores:running", NULL, NULL, NULL, 0, 1,
+    st->rpc = nc_rpc_establishpush_onchange("ietf-datastores:running", "/edit1:*", NULL, NULL, 0, 1,
             NULL, NC_PARAMTYPE_CONST);
     st->msgtype = nc_send_rpc(st->nc_sess, st->rpc, 1000, &st->msgid);
     assert_int_equal(st->msgtype, NC_MSG_RPC);
@@ -725,7 +700,7 @@ test_sync_on_start(void **state)
 static int
 setup_test_sync_on_start_non_empty(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data;
 
     /* Put some data into the datastore */
@@ -739,12 +714,12 @@ setup_test_sync_on_start_non_empty(void **state)
 static void
 test_sync_on_start_non_empty(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *template;
     char *ntf;
 
     /* Establish on-change push with sync on start */
-    st->rpc = nc_rpc_establishpush_onchange("ietf-datastores:running", NULL, NULL, NULL, 0, 1,
+    st->rpc = nc_rpc_establishpush_onchange("ietf-datastores:running", "/edit1:*", NULL, NULL, 0, 1,
             NULL, NC_PARAMTYPE_CONST);
     st->msgtype = nc_send_rpc(st->nc_sess, st->rpc, 1000, &st->msgid);
     assert_int_equal(st->msgtype, NC_MSG_RPC);
@@ -769,12 +744,12 @@ test_sync_on_start_non_empty(void **state)
 static void
 test_resync(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *template;
     char *ntf;
 
     /* Establish on-change push */
-    st->rpc = nc_rpc_establishpush_onchange("ietf-datastores:running", NULL, NULL, NULL, 0, 0,
+    st->rpc = nc_rpc_establishpush_onchange("ietf-datastores:running", "/edit1:*", NULL, NULL, 0, 0,
             NULL, NC_PARAMTYPE_CONST);
     st->msgtype = nc_send_rpc(st->nc_sess, st->rpc, 1000, &st->msgid);
     assert_int_equal(st->msgtype, NC_MSG_RPC);
@@ -807,12 +782,12 @@ test_resync(void **state)
 static void
 test_resync_id_reset(void **state)
 {
-    struct np_test *st = *state;
+    struct np2_test *st = *state;
     const char *data, *template;
     char *ntf;
 
     /* Establish on-change push */
-    st->rpc = nc_rpc_establishpush_onchange("ietf-datastores:running", NULL, NULL, NULL, 0, 0,
+    st->rpc = nc_rpc_establishpush_onchange("ietf-datastores:running", "/edit1:*", NULL, NULL, 0, 0,
             NULL, NC_PARAMTYPE_CONST);
     st->msgtype = nc_send_rpc(st->nc_sess, st->rpc, 1000, &st->msgid);
     assert_int_equal(st->msgtype, NC_MSG_RPC);
